import dash
import dash_table
import pandas as pd
import dash_design_kit as ddk

df = pd.read_csv("test.csv")
app = dash.Dash(__name__)

app.layout = ddk.App(show_editor=True, children=[
    ddk.DataTable(
       id='table',
       columns=[{"name": i, "id": i} for i in df.columns],
       data=df.to_dict('records'),
       editable=True
   )
])

if __name__ == '__main__':
    app.run_server(debug=True)


import gzip

from re_common.baselibrary.utils.basefile import BaseFile


class BaseGzip(object):

    def __init__(self, bufsize, fin=None, fout=None):
        self.bufsize = bufsize
        self.fin = fin
        self.fout = fout

    def compress(self, src, dst):
        """
        压缩文件
        :param src:
        :param dst:
        :return:
        """
        self.fin = open(src, 'rb')
        self.fout = gzip.open(dst, 'wb')

        self.__in2out()

    def decompress(self, gzFile, dst):
        """
        解压文件
        :param gzFile:
        :param dst:
        :return:
        """
        self.fin = gzip.open(gzFile, 'rb')
        self.fout = open(dst, 'wb')

        self.__in2out()

    def __in2out(self):
        while True:
            buf = self.fin.read(self.bufsize)
            if len(buf) < 1:
                break
            self.fout.write(buf)

        self.fin.close()
        self.fout.close()

    def read_gz_file(self, file):
        with gzip.open(file, 'r') as f:
            for lineb in f:
                line = lineb.decode()
                yield line.strip()

    @classmethod
    def get_gz_line_num(self, file):
        i = 0
        if BaseFile.is_file_exists(file):
            with gzip.open(file, 'rb') as f:
                for i, l in enumerate(f):
                    pass
        return i



import aiomysql

from re_common.baselibrary import IniConfig
from re_common.baselibrary.readconfig.toml_config import TomlConfig


class BaseAioMysql(object):

    def __init__(self):
        self.conn: aiomysql.connection.Connection

    def set_conn_dicts(self, dicts):
        self.host = dicts["host"]
        self.port = dicts["port"]
        self.user = dicts["user"]
        self.password = dicts["password"]
        self.db = dicts["db"]
        self.loop = dicts["loop"]
        return self

    def set_conn_ini(self, config_ini, secs='mysql'):
        """
        使用配置文件连接
        :param config_ini:
        :return:
        """
        dicts = IniConfig(config_ini).builder().get_config_dict()
        self.set_conn_dicts(dicts[secs])
        return self

    def set_conn_toml(self, config_toml, secs=None):
        """
        使用配置文件连接
        :param config_ini:
        :return:
        """
        dicts = TomlConfig(config_toml).get_config_dicts()
        if secs:
            dicts = dicts[secs]
        self.set_conn_dicts(dicts)
        return self

    def set_loop(self, loop):
        self.loop = loop
        return self

    async def conn_mysql(self):
        self.conn = await aiomysql.connect(
            host=self.host, port=self.port,
            user=self.user, password=self.password,
            db=self.db, loop=self.loop)
        return self

    async def exec_sql(self, sql, args=None):
        await self.conn_mysql()
        cur = await self.conn.cursor()
        # count 已生成的受影响的行数
        count = await cur.execute(sql, args)
        # https://aiomysql.readthedocs.io/en/latest/cursors.html?highlight=description#Cursor.description
        # 此只读属性是7个项目的序列的序列。
        # print(cur.description)
        # 返回的查询值
        r = await cur.fetchall()
        await cur.close()
        return r

    async def get_pool(self):
        self.pool = await aiomysql.create_pool(
            host=self.host, port=self.port,
            user=self.user, password=self.password,
            db=self.db, loop=self.loop,
            autocommit=False)

    async def pool_exec(self, sql, arg=None):
        await self.get_pool()
        with (await self.pool) as conn:
            cur = await conn.cursor()
            await cur.execute(sql, arg)
            (r,) = await cur.fetchone()
        return r

    def close_conn(self):
        self.conn.close()

    async def close_pool(self):
        await self.pool.wait_closed()
        self.pool.close()



import base64
import hashlib

"""
VIP编码lngid生成
"""


class BaseLngid(object):
    def __int__(self):
        pass

    def BaseEncodeID(self, strRaw):
        r""" 自定义base编码 """

        strEncode = base64.b32encode(strRaw.encode('utf8')).decode('utf8')

        if strEncode.endswith('======'):
            strEncode = '%s%s' % (strEncode[0:-6], '0')
        elif strEncode.endswith('===='):
            strEncode = '%s%s' % (strEncode[0:-4], '1')
        elif strEncode.endswith('==='):
            strEncode = '%s%s' % (strEncode[0:-3], '8')
        elif strEncode.endswith('='):
            strEncode = '%s%s' % (strEncode[0:-1], '9')

        table = str.maketrans('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'ZYXWVUTSRQPONMLKJIHGFEDCBA9876543210')
        strEncode = strEncode.translate(table)

        return strEncode

    def BaseDecodeID(self, strEncode):
        r""" 自定义base解码 """

        table = str.maketrans('ZYXWVUTSRQPONMLKJIHGFEDCBA9876543210', '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ')
        strEncode = strEncode.translate(table)

        if strEncode.endswith('0'):
            strEncode = '%s%s' % (strEncode[0:-1], '======')
        elif strEncode.endswith('1'):
            strEncode = '%s%s' % (strEncode[0:-1], '====')
        elif strEncode.endswith('8'):
            strEncode = '%s%s' % (strEncode[0:-1], '===')
        elif strEncode.endswith('9'):
            strEncode = '%s%s' % (strEncode[0:-1], '=')

        strRaw = base64.b32decode(strEncode.encode('utf8')).decode('utf8')

        return strRaw

    def GetLngid(self, sub_db_id, rawid, case_insensitive=False):
        """
        :param sub_db_id:
        :param rawid:
        由 sub_db_id 和 rawid 得到 lngid。
        :param case_insensitive: 标识源网站的 rawid 是否区分大小写
        :return: lngid
        """
        uppercase_rawid = ''  # 大写版 rawid
        if case_insensitive:  # 源网站的 rawid 区分大小写
            for ch in rawid:
                if ch.upper() == ch:
                    uppercase_rawid += ch
                else:
                    uppercase_rawid += ch.upper() + '_'
        else:
            uppercase_rawid = rawid.upper()

        limited_id = uppercase_rawid  # 限长ID
        if len(uppercase_rawid) > 20:
            limited_id = hashlib.md5(uppercase_rawid.encode('utf8')).hexdigest().upper()
        else:
            limited_id = self.BaseEncodeID(uppercase_rawid)

        lngid = sub_db_id + limited_id

        return lngid

    def getDoiid(self, doi, case_insensitive=False):
        if case_insensitive is False:
            doi = doi.upper()
        limited_id = self.BaseEncodeID(doi)
        if len(limited_id) > 240:
            limited_id = hashlib.md5(doi.encode('utf8')).hexdigest().upper()
        return limited_id

    def GetRawid(self, limited_id, case_insensitive=False):
        try:
            uppercase_rawid = self.BaseDecodeID(limited_id)
            if case_insensitive:
                str_ = "_"
                uppercase_rawid_list = list(uppercase_rawid)
                for num, li in enumerate(uppercase_rawid_list):
                    if li == str_:
                        old_str = "".join(uppercase_rawid_list[num - 1:num + 1])
                        uppercase_rawid = uppercase_rawid.replace(old_str, uppercase_rawid_list[num - 1].lower())
        except Exception as e:
            raise Exception("长度超过20，不可逆")

        return uppercase_rawid



import logging

from re_common.baselibrary.utils.basedir import BaseDir
from re_common.baselibrary import MLogger
from re_common.baselibrary.utils.basefile import BaseFile


def test_mlogger_streamlogger():
    """
    只打印数据不计入文件
    pytest mylogger_test.py::test_mlogger_streamlogger -s
    :return:
    """
    streamlogger = MLogger().streamlogger
    streamlogger.setLevel(logging.DEBUG)
    streamlogger.info("mesg info")
    streamlogger.warning("msg warning")
    streamlogger.warn("msg warn")
    streamlogger.error("msg error")
    streamlogger.debug("msg debug")
    streamlogger.critical("msg critical")
    streamlogger.exception("msg exception")


def test_mlogger_filelogger():
    """
    测试将日志写入文件中　文件名以日期自动生成　如20200211.log
    pytest mylogger_test.py::test_mlogger_filelogger -s
    文件流的logger
    :param filedir:
    :return:
    """
    filedir = BaseDir.get_file_dir_absolute(__file__)
    filelogger = MLogger(filedir).filelogger
    filelogger.setLevel(logging.DEBUG)
    filelogger.info("mesg info")
    filelogger.warning("msg warning")
    filelogger.warn("msg warn")
    filelogger.error("msg error")
    filelogger.debug("msg debug")
    filelogger.critical("msg critical")
    filelogger.exception("msg exception")


def test_mlogger_allloger():
    """
    测试将日志写入文件中 有一个以日期的总文件　以及每个级别的分文件
    pytest mylogger_test.py::test_mlogger_allloger -s
    文件流的logger
    :param filedir:
    :return:
    """
    filedir = BaseDir.get_file_dir_absolute(__file__)
    filedir = BaseFile.get_new_path(filedir, "log")
    BaseDir.create_dir(filedir)
    filelogger = MLogger(filedir).alllogger
    filelogger.setLevel(logging.DEBUG)
    filelogger.info("mesg info")
    filelogger.warning("msg warning")
    filelogger.warn("msg warn")
    filelogger.error("msg error")
    filelogger.debug("msg debug")
    filelogger.critical("msg critical")
    filelogger.exception("msg exception")


def test_mlogger_rotatingfilelogger():
    """
    测试将日志写入文件中　文件名以日期自动生成　如20200211.log
    pytest mylogger_test.py::test_mlogger_rotatingfilelogger -s
    文件流的logger
    :param filedir:
    :return:
    """
    filedir = BaseDir.get_file_dir_absolute(__file__)
    filelogger = MLogger(filedir).RotatingFileLogger(filedir, None, maxBytes=1*1024*1024,
                                                               backupCount=10)
    filelogger.setLevel(logging.DEBUG)
    while True:
        print("**************")
        filelogger.info("mesg info")
        filelogger.warning("msg warning")
        filelogger.warn("msg warn")
        filelogger.error("msg error")
        filelogger.debug("msg debug")
        filelogger.critical("msg critical")
        filelogger.exception("msg exception")

test_mlogger_rotatingfilelogger()


import random

from faker import Faker


# https://www.jianshu.com/p/20e41fc65dc8

class MFaker(Faker):
    # 生成名字
    m_name = "name"
    # 生成地址
    m_address = "address"
    # 生成国家
    m_country = "country"
    # 生成省份
    m_province = "province"
    # 生成市　县
    m_city_suffix = "city_suffix"

    # 以下结构都可以用　py_para　结构的参数
    # 生成dict
    m_pydict = "pydict"
    # 生成list
    m_pylist = "pylist"
    # 生成iterable
    m_pyiterable = "pyiterable"
    # 生成set
    m_pyset = "pyset"
    # 生成tuple
    m_pytuple = "pytuple"

    def create_data(self, types, **kwargs):
        """
        :param types: 是上面的类型
        :param kwargs: 类型的参数
        :return:
        """
        if kwargs:
            print(kwargs)
            return getattr(self, types)(**kwargs)
        return getattr(self, types)()

    def py_para(self, num=10):
        """
        python 集合参数　适用于python 原生集合
        pydict pylist 的参数
        :return:
        """
        return {"nb_elements": num, "variable_nb_elements": True}

    def get_page(self):
        return random.randint(1, 150)








import pypyodbc

from re_common.baselibrary.utils.baseodbc import BaseODBC
from re_common.facade.sqlite3facade import Sqlite3Utiles


class MdbDb3(object):
    """
    mdb和db3的相互转换
    """

    def __init__(self):
        self.MdbFile = ''
        self.db3file = ''
        self.baseodbc = None
        self.sqliteutils = None

    def set_mdb(self, MdbFile):
        self.MdbFile = MdbFile
        self.baseodbc = BaseODBC(self.MdbFile)
        return self

    def set_db3(self, db3_file):
        self.db3file = db3_file
        self.sqliteutils = Sqlite3Utiles().Sqlite3DBConnectFromFilePath(self.db3file)
        return self

    def get_db3_table_name(self):
        """
        获取db3的table表明
        :return:
        """
        lists_tablename = []
        for tables in self.sqliteutils.sqllite3.get_table_name():
            table = tables[0]
            lists_tablename.append(table)
        return lists_tablename

    def get_db3_fields(self, tablename):
        self.sqliteutils.sqllite3.set_result_dict()
        result = self.sqliteutils.sqllite3.get_all_field_info(tablename)
        print(result)


if __name__ == "__main__":
    md3 = MdbDb3().set_db3(r'C:\Users\xuzhu\Desktop\cnkijournallist_1628228383.4636774.db3')
    for tablename in md3.get_db3_table_name():
        md3.get_db3_fields(tablename)



from re_common.baselibrary.database.msqlite3 import Sqlite3
from re_common.baselibrary.utils.basedir import BaseDir

onepath = r"F:\db32\test\base_obj_meta_a_qkwx.cnki_qk.20200527_000.db3"
for file in BaseDir.get_dir_all_files(r"F:\db32\cnki"):
    Sqlite3.sqlite3_merge(onepath, file, tablename="base_obj_meta_a")


###########################################
# 同项目调用基础包
import os
import shutil
import sys

filepath = os.path.abspath(__file__)
pathlist = filepath.split(os.sep)
pathlist = pathlist[:-4]
TopPath = os.sep.join(pathlist)
sys.path.insert(0, TopPath)
print(TopPath)
############################################

from re_common.baselibrary.utils.basedir import BaseDir
from re_common.baselibrary.utils.basefile import BaseFile

source_dir = r"\\192.168.31.169\down_data\cnki_hy_quanwen\20200530\vcode"
dst_dir = r"F:\fun2\jpg"
BaseDir.create_dir(dst_dir)
log_path = r"F:\fun2\log.txt"
count = 0
for file in BaseDir.get_dir_all_files(source_dir):
    print(file)
    count = count + 1
    try:
        filename = BaseFile.get_file_name(file)
        filepath = BaseDir.get_file_dir_absolute(file)
        new_sub_dir = filepath.replace(source_dir, "")
        new_dir = dst_dir + new_sub_dir
        BaseDir.create_dir(new_dir)
        new_path = new_dir + "\\" + filename
        shutil.copyfile(file, new_path)
    except:
        print("err:" + file)
        BaseFile.single_add_file(log_path, file)
    # if count % 10000 == 1:
    #     print(count)
    print(count)



import time

from re_common.baselibrary.utils.basefile import BaseFile

with open(r"\\192.168.31.171\caiji\test.txt", 'w', encoding="utf-8") as f:
    while True:
        f.write("test\n")
        time.sleep(10)
        print("1")


from abc import ABCMeta, abstractmethod

from re_common.baselibrary.database.moudle import SqlMoudle, Sqlite3Moudle


class MysqlBuilderAbstract:
    __metaclass__ = ABCMeta

    @abstractmethod
    def build_server_address(self, host):
        pass

    @abstractmethod
    def build_username(self, user):
        pass

    @abstractmethod
    def build_password(self, passwd):
        pass

    @abstractmethod
    def build_port(self, port):
        pass

    @abstractmethod
    def use_db(self, db):
        pass

    @abstractmethod
    def build_chart(self, chartset):
        pass

    @abstractmethod
    def get_moudle(self):
        pass


class Sqlite3BuilderAbstract:
    __metaclass__ = ABCMeta

    @abstractmethod
    def build_file_path(self, dbpath):
        pass

    @abstractmethod
    def build_isolation_level(self):
        pass


class MysqlBuilder(MysqlBuilderAbstract):
    def __init__(self, configfile, configname, keytransformdicts=None):
        self.sqlmoudle = SqlMoudle()

        from re_common.baselibrary.readconfig.ini_config import IniConfig
        self.ic = IniConfig(configfile).builder()
        self.configfile = configfile
        self.configname = configname
        # 使用key转换器转换key
        self.keytransformdicts = keytransformdicts
        """
        python2
            self.config.readfp(open(configfile, 'rb'))
        python3
            self.config.read_file(open(configfile, 'r'))
        """

    def build_port(self, port):
        self.sqlmoudle.port = int(self.ic.get_value(self.configname, port))

    def build_server_address(self, host):
        self.sqlmoudle.host = self.ic.get_value(self.configname, host)

    def build_password(self, passwd):
        self.sqlmoudle.passwd = self.ic.get_value(self.configname, passwd)

    def use_db(self, db):
        self.sqlmoudle.db = self.ic.get_value(self.configname, db)

    def build_username(self, user):
        self.sqlmoudle.user = self.ic.get_value(self.configname, user)

    def build_chart(self, chartset):
        self.sqlmoudle.charset = self.ic.get_value(self.configname, chartset)

    def get_moudle(self):
        return self.sqlmoudle

    def build_all(self):
        if not self.keytransformdicts:
            self.keytransformdicts = {"port": "port", "host": "host", "passwd": "passwd", "user": "user", "db": "db",
                                      "chartset": "chartset"}
        self.build_port(self.keytransformdicts["port"])
        self.build_server_address(self.keytransformdicts["host"])
        self.build_password(self.keytransformdicts["passwd"])
        self.build_username(self.keytransformdicts["user"])
        self.use_db(self.keytransformdicts["db"])
        self.build_chart(self.keytransformdicts["chartset"])
        return self

    def get_tuples(self):
        return self.sqlmoudle.host, self.sqlmoudle.user, self.sqlmoudle.passwd, self.sqlmoudle.port


class Sqlite3Builder(Sqlite3BuilderAbstract):
    def __init__(self, configfile="", sec=""):
        self.sqlite3moudle = Sqlite3Moudle()
        from re_common.baselibrary.readconfig.ini_config import IniConfig
        self.ic = IniConfig(configfile).builder()
        self.configfile = configfile
        self.sec = sec

    def build_file_path(self, dbpath='', opt="dbpath"):
        if dbpath:
            self.sqlite3moudle.database = dbpath
        else:
            self.sqlite3moudle.database = self.ic.get_value(self.sec, opt)
        return self

    def build_timeout(self, timeout):
        self.sqlite3moudle.timeout = timeout
        return self

    def build_isolation_level(self):
        pass

    def build_all(self):
        self.build_file_path()
        self.build_isolation_level()
        return self

    def get_moudle(self):
        return self.sqlite3moudle



# coding=utf-8
import hashlib
import json
import time

import requests

FATEA_PRED_URL = "http://pred.fateadm.com"


def LOG(log):
    # 不需要测试时，注释掉日志就可以了
    print(log)
    log = None


class TmpObj():
    def __init__(self):
        self.value = None


class Rsp():
    def __init__(self):
        self.ret_code = -1
        self.cust_val = 0.0
        self.err_msg = "succ"
        self.pred_rsp = TmpObj()

    def ParseJsonRsp(self, rsp_data):
        if rsp_data is None:
            self.err_msg = "http request failed, get rsp Nil data"
            return
        jrsp = json.loads(rsp_data)
        self.ret_code = int(jrsp["RetCode"])
        self.err_msg = jrsp["ErrMsg"]
        self.request_id = jrsp["RequestId"]
        if self.ret_code == 0:
            rslt_data = jrsp["RspData"]
            if rslt_data is not None and rslt_data != "":
                jrsp_ext = json.loads(rslt_data)
                if "cust_val" in jrsp_ext:
                    data = jrsp_ext["cust_val"]
                    self.cust_val = float(data)
                if "result" in jrsp_ext:
                    data = jrsp_ext["result"]
                    self.pred_rsp.value = data


def CalcSign(pd_id, passwd, timestamp):
    md5 = hashlib.md5()
    md5.update((timestamp + passwd).encode())
    csign = md5.hexdigest()

    md5 = hashlib.md5()
    md5.update((pd_id + timestamp + csign).encode())
    csign = md5.hexdigest()
    return csign


def CalcCardSign(cardid, cardkey, timestamp, passwd):
    md5 = hashlib.md5()
    md5.update(passwd + timestamp + cardid + cardkey)
    return md5.hexdigest()


def HttpRequest(url, body_data, img_data=""):
    rsp = Rsp()
    post_data = body_data
    files = {
        'img_data': ('img_data', img_data)
    }
    header = {
        'User-Agent': 'Mozilla/5.0',
    }
    rsp_data = requests.post(url, post_data, files=files, headers=header)
    rsp.ParseJsonRsp(rsp_data.text)
    return rsp


class FateadmApi():
    # API接口调用类
    # 参数（appID，appKey，pdID，pdKey）
    def __init__(self, app_id, app_key, pd_id, pd_key):
        self.app_id = app_id
        if app_id is None:
            self.app_id = ""
        self.app_key = app_key
        self.pd_id = pd_id
        self.pd_key = pd_key
        self.host = FATEA_PRED_URL

    def SetHost(self, url):
        self.host = url

    #
    # 查询余额
    # 参数：无
    # 返回值：
    #   rsp.ret_code：正常返回0
    #   rsp.cust_val：用户余额
    #   rsp.err_msg：异常时返回异常详情
    #
    def QueryBalc(self):
        tm = str(int(time.time()))
        sign = CalcSign(self.pd_id, self.pd_key, tm)
        param = {
            "user_id": self.pd_id,
            "timestamp": tm,
            "sign": sign
        }
        url = self.host + "/api/custval"
        rsp = HttpRequest(url, param)
        if rsp.ret_code == 0:
            LOG("query succ ret: {} cust_val: {} rsp: {} pred: {}".format(rsp.ret_code, rsp.cust_val, rsp.err_msg,
                                                                          rsp.pred_rsp.value))
        else:
            LOG("query failed ret: {} err: {}".format(rsp.ret_code, rsp.err_msg.encode('utf-8')))
        return rsp

    #
    # 查询网络延迟
    # 参数：pred_type:识别类型
    # 返回值：
    #   rsp.ret_code：正常返回0
    #   rsp.err_msg： 异常时返回异常详情
    #
    def QueryTTS(self, pred_type):
        tm = str(int(time.time()))
        sign = CalcSign(self.pd_id, self.pd_key, tm)
        param = {
            "user_id": self.pd_id,
            "timestamp": tm,
            "sign": sign,
            "predict_type": pred_type,
        }
        if self.app_id != "":
            #
            asign = CalcSign(self.app_id, self.app_key, tm)
            param["appid"] = self.app_id
            param["asign"] = asign
        url = self.host + "/api/qcrtt"
        rsp = HttpRequest(url, param)
        if rsp.ret_code == 0:
            LOG("query rtt succ ret: {} request_id: {} err: {}".format(rsp.ret_code, rsp.request_id, rsp.err_msg))
        else:
            LOG("predict failed ret: {} err: {}".format(rsp.ret_code, rsp.err_msg.encode('utf-8')))
        return rsp

    #
    # 识别验证码
    # 参数：pred_type:识别类型  img_data:图片的数据
    # 返回值：
    #   rsp.ret_code：正常返回0
    #   rsp.request_id：唯一订单号
    #   rsp.pred_rsp.value：识别结果
    #   rsp.err_msg：异常时返回异常详情
    #
    def Predict(self, pred_type, img_data, head_info=""):
        tm = str(int(time.time()))
        sign = CalcSign(self.pd_id, self.pd_key, tm)
        param = {
            "user_id": self.pd_id,
            "timestamp": tm,
            "sign": sign,
            "predict_type": pred_type,
            "up_type": "mt"
        }
        if head_info is not None or head_info != "":
            param["head_info"] = head_info
        if self.app_id != "":
            #
            asign = CalcSign(self.app_id, self.app_key, tm)
            param["appid"] = self.app_id
            param["asign"] = asign
        url = self.host + "/api/capreg"
        files = img_data
        rsp = HttpRequest(url, param, files)
        if rsp.ret_code == 0:
            LOG("predict succ ret: {} request_id: {} pred: {} err: {}".format(rsp.ret_code, rsp.request_id,
                                                                              rsp.pred_rsp.value, rsp.err_msg))
        else:
            LOG("predict failed ret: {} err: {}".format(rsp.ret_code, rsp.err_msg))
            if rsp.ret_code == 4003:
                # lack of money
                LOG("cust_val <= 0 lack of money, please charge immediately")
        return rsp

    #
    # 从文件进行验证码识别
    # 参数：pred_type;识别类型  file_name:文件名
    # 返回值：
    #   rsp.ret_code：正常返回0
    #   rsp.request_id：唯一订单号
    #   rsp.pred_rsp.value：识别结果
    #   rsp.err_msg：异常时返回异常详情
    #
    def PredictFromFile(self, pred_type, file_name, head_info=""):
        with open(file_name, "rb") as f:
            data = f.read()
        return self.Predict(pred_type, data, head_info=head_info)

    #
    # 识别失败，进行退款请求
    # 参数：request_id：需要退款的订单号
    # 返回值：
    #   rsp.ret_code：正常返回0
    #   rsp.err_msg：异常时返回异常详情
    #
    # 注意:
    #    Predict识别接口，仅在ret_code == 0时才会进行扣款，才需要进行退款请求，否则无需进行退款操作
    # 注意2:
    #   退款仅在正常识别出结果后，无法通过网站验证的情况，请勿非法或者滥用，否则可能进行封号处理
    #
    def Justice(self, request_id):
        if request_id == "":
            #
            return
        tm = str(int(time.time()))
        sign = CalcSign(self.pd_id, self.pd_key, tm)
        param = {
            "user_id": self.pd_id,
            "timestamp": tm,
            "sign": sign,
            "request_id": request_id
        }
        url = self.host + "/api/capjust"
        rsp = HttpRequest(url, param)
        if rsp.ret_code == 0:
            LOG("justice succ ret: {} request_id: {} pred: {} err: {}".format(rsp.ret_code, rsp.request_id,
                                                                              rsp.pred_rsp.value, rsp.err_msg))
        else:
            LOG("justice failed ret: {} err: {}".format(rsp.ret_code, rsp.err_msg.encode('utf-8')))
        return rsp

    #
    # 充值接口
    # 参数：cardid：充值卡号  cardkey：充值卡签名串
    # 返回值：
    #   rsp.ret_code：正常返回0
    #   rsp.err_msg：异常时返回异常详情
    #
    def Charge(self, cardid, cardkey):
        tm = str(int(time.time()))
        sign = CalcSign(self.pd_id, self.pd_key, tm)
        csign = CalcCardSign(cardid, cardkey, tm, self.pd_key)
        param = {
            "user_id": self.pd_id,
            "timestamp": tm,
            "sign": sign,
            'cardid': cardid,
            'csign': csign
        }
        url = self.host + "/api/charge"
        rsp = HttpRequest(url, param)
        if rsp.ret_code == 0:
            LOG("charge succ ret: {} request_id: {} pred: {} err: {}".format(rsp.ret_code, rsp.request_id,
                                                                             rsp.pred_rsp.value, rsp.err_msg))
        else:
            LOG("charge failed ret: {} err: {}".format(rsp.ret_code, rsp.err_msg.encode('utf-8')))
        return rsp

    ##
    # 充值，只返回是否成功
    # 参数：cardid：充值卡号  cardkey：充值卡签名串
    # 返回值： 充值成功时返回0
    ##
    def ExtendCharge(self, cardid, cardkey):
        return self.Charge(cardid, cardkey).ret_code

    ##
    # 调用退款，只返回是否成功
    # 参数： request_id：需要退款的订单号
    # 返回值： 退款成功时返回0
    #
    # 注意:
    #    Predict识别接口，仅在ret_code == 0时才会进行扣款，才需要进行退款请求，否则无需进行退款操作
    # 注意2:
    #   退款仅在正常识别出结果后，无法通过网站验证的情况，请勿非法或者滥用，否则可能进行封号处理
    ##
    def JusticeExtend(self, request_id):
        return self.Justice(request_id).ret_code

    ##
    # 查询余额，只返回余额
    # 参数：无
    # 返回值：rsp.cust_val：余额
    ##
    def QueryBalcExtend(self):
        rsp = self.QueryBalc()
        return rsp.cust_val

    ##
    # 从文件识别验证码，只返回识别结果
    # 参数：pred_type;识别类型  file_name:文件名
    # 返回值： rsp.pred_rsp.value：识别的结果
    ##
    def PredictFromFileExtend(self, pred_type, file_name, head_info=""):
        rsp = self.PredictFromFile(pred_type, file_name, head_info)
        return rsp.pred_rsp.value

    ##
    # 识别接口，只返回识别结果
    # 参数：pred_type:识别类型  img_data:图片的数据
    # 返回值： rsp.pred_rsp.value：识别的结果
    ##
    def PredictExtend(self, pred_type, img_data, head_info=""):
        rsp = self.Predict(pred_type, img_data, head_info)
        return rsp.pred_rsp.value


#
# def TestFunc():
#     pd_id = "120292"  # 用户中心页可以查询到pd信息
#     pd_key = "i3M9C5vG7nWqAyP9xW2aZke9utdZsJRC"
#     app_id = "320292"  # 开发者分成用的账号，在开发者中心可以查询到
#     app_key = "c7Op85NwodVYZjZ80pPp/2QTkEAPouGT"
#     # 识别类型，
#     # 具体类型可以查看官方网站的价格页选择具体的类型，不清楚类型的，可以咨询客服
#     pred_type = "30400"
#     api = FateadmApi(app_id, app_key, pd_id, pd_key)
#     # 查询余额
#     balance = api.QueryBalcExtend()  # 直接返余额
#     # api.QueryBalc()
#
#     # 通过文件形式识别：
#     file_name = "img.jpg"
#     # 多网站类型时，需要增加src_url参数，具体请参考api文档: http://docs.fateadm.com/web/#/1?page_id=6
#     # result =  api.PredictFromFileExtend(pred_type,file_name)   # 直接返回识别结果
#     rsp = api.PredictFromFile(pred_type, file_name)  # 返回详细识别结果
#
#     '''
#     # 如果不是通过文件识别，则调用Predict接口：
#     # result 			= api.PredictExtend(pred_type,data)   	# 直接返回识别结果
#     rsp             = api.Predict(pred_type,data)				# 返回详细的识别结果
#     '''
#
#     just_flag = False
#     if just_flag:
#         if rsp.ret_code == 0:
#             # 识别的结果如果与预期不符，可以调用这个接口将预期不符的订单退款
#             # 退款仅在正常识别出结果后，无法通过网站验证的情况，请勿非法或者滥用，否则可能进行封号处理
#             api.Justice(rsp.request_id)
#
#     # card_id         = "123"
#     # card_key        = "123"
#     # 充值
#     # api.Charge(card_id, card_key)
#     LOG("print in testfunc")
#
#
# if __name__ == "__main__":
#     TestFunc()

class UseFateadm(object):
    def __init__(self, pd_id="120292", pd_key="i3M9C5vG7nWqAyP9xW2aZke9utdZsJRC", app_id="320292",
                 app_key="c7Op85NwodVYZjZ80pPp/2QTkEAPouGT"):
        self.pd_id = pd_id  # 用户中心页可以查询到pd信息
        self.pd_key = pd_key
        self.app_id = app_id  # 开发者分成用的账号，在开发者中心可以查询到
        self.app_key = app_key
        # 具体类型可以查看官方网站的价格页选择具体的类型，不清楚类型的，可以咨询客服
        self.pred_type = "30400"
        self.api = FateadmApi(self.app_id, self.app_key, self.pd_id, self.pd_key)

    def set_pred_type(self, pred_type):
        self.pred_type = pred_type

    def get_balance(self):
        # 查询余额
        balance = self.api.QueryBalcExtend()  # 直接返余额
        return balance

    def get_queryBalc(self):
        """
        # 查询余额
        # 参数：无
        # 返回值：
        #   rsp.ret_code：正常返回0
        #   rsp.cust_val：用户余额
        #   rsp.err_msg：异常时返回异常详情
        #
        :return:
        """
        self.api.QueryBalc()

    def PredictFromFileExtend(self, file_name, pred_type=None):
        if pred_type == None:
            pred_type = self.pred_type
        # 多网站类型时，需要增加src_url参数，具体请参考api文档: http://docs.fateadm.com/web/#/1?page_id=6
        result = self.api.PredictFromFileExtend(pred_type, file_name)  # 直接返回识别结果
        return result

    def PredictFromFile(self, file_name, pred_type=None):  # 返回详细识别结果
        if pred_type == None:
            pred_type = self.pred_type
        rsp = self.api.PredictFromFile(pred_type, file_name)  # 返回详细识别结果
        return rsp

    def PredictExtend(self, data, pred_type=None):
        if pred_type == None:
            pred_type = self.pred_type
        # 多网站类型时，需要增加src_url参数，具体请参考api文档: http://docs.fateadm.com/web/#/1?page_id=6
        result = self.api.PredictExtend(pred_type, data)  # 直接返回识别结果
        return result

    def Predict(self, data, pred_type=None):  # 返回详细识别结果
        if pred_type == None:
            pred_type = self.pred_type
        rsp = self.api.Predict(pred_type, data)  # 返回详细的识别结果
        return rsp

    def Justice(self, rsp):
        if rsp.ret_code == 0:
            # 识别的结果如果与预期不符，可以调用这个接口将预期不符的订单退款
            # 退款仅在正常识别出结果后，无法通过网站验证的情况，请勿非法或者滥用，否则可能进行封号处理
            self.api.Justice(rsp.request_id)

    def Charge(self, card_id, card_key):
        """
        # 充值接口
        # 参数：cardid：充值卡号  cardkey：充值卡签名串
        # 返回值：
        #   rsp.ret_code：正常返回0
        #   rsp.err_msg：异常时返回异常详情
        :param card_id:
        :param card_key:
        :return:
        """
        self.api.Charge(card_id, card_key)



import base64
import hashlib
import os

from re_common.v2.baselibrary.business_utils.baseencodeid import BaseLngid

import os
import base64
import hashlib

"""
DOI-文件路径 转换工具

设计目标：
1. 将任意DOI字符串转换为可逆、稳定的文件路径
2. 提供高效的目录分散方案（65,536个子目录）
3. 支持带文件扩展名的存储
4. 完全可逆转换

工作原理：
1. DOI编码：
   - 使用URL安全的Base64编码（RFC 3548）
   - 移除Base64填充的'='字符
   - 文件名长度 ≈ 原始DOI长度 × 4/3

2. 目录分散：
   - 使用MD5哈希创建两级目录结构
   - 目录层级：/MD5[0:2]/MD5[2:4]/
   - 支持65,536个目录（256×256），每目录约1,525个文件（假设10亿文件）

3. 扩展名处理：
   - 保持原始扩展名不变
   - 解码时自动忽略扩展名

典型转换示例：
DOI: "10.1000/xyz123" -> 路径: "a1/b2/QTMuMTAwMC94eXoxMjM.pdf"
路径: "a1/b2/QTMuMTAwMC94eXoxMjM.pdf" -> DOI: "10.1000/xyz123"
"""

base_lngid = BaseLngid()


# 以后需要启用
def doi_to_path(doi: str, ext: str = "") -> str:
    """
    将 DOI 转换为可逆的存储路径：
    1. 对 DOI 进行 URL 安全的 Base64 编码（可逆）
    2. 生成 DOI 的 MD5 哈希用于目录分散
    3. 目录结构：MD5前2字符/次2字符/
    4. 文件名：Base64编码的DOI + 扩展名

    Args:
        doi: 文件 DOI 标识符
        ext: 文件扩展名（如 '.pdf'）

    Returns:
        相对文件路径（如 'a1/b2/QTMuMTAwMC94eXoxMjM=.pdf'）
    """
    # URL安全的Base64编码（可逆）
    doi_b64 = base64.urlsafe_b64encode(doi.encode("utf-8")).decode("ascii").rstrip("=")

    # 生成MD5哈希用于目录分配
    hash_md5 = hashlib.md5(doi.encode("utf-8")).hexdigest()
    dir_level1 = hash_md5[0:2]
    dir_level2 = hash_md5[2:4]

    return os.path.join(dir_level1, dir_level2, f"{doi_b64}{ext}")


# 以后需要启用
def path_to_doi(path: str) -> str:
    """
    从文件路径反推原始DOI
    Args:
        path: 文件路径（如 'a1/b2/QTMuMTAwMC94eXoxMjM=.pdf'）

    Returns:
        原始DOI字符串
    """
    # 提取文件名并移除扩展名
    filename = os.path.basename(path)
    base_name = os.path.splitext(filename)[0]

    # 补齐Base64填充字符
    padding = 4 - (len(base_name) % 4)
    if padding != 4:  # 不需要补齐
        base_name += "=" * padding

    # Base64解码还原DOI
    return base64.urlsafe_b64decode(base_name.encode("ascii")).decode("utf-8")


def doi_to_dir(doi):
    """生成文件的存储路径和可解码的文件名

    Args:
        doi (str): 文件的唯一DOI标识

    Returns:
        str: 文件相对路径，如 "ab/cd/Base64EncodedFileName"
    """
    # 计算DOI的MD5哈希
    hash_md5 = hashlib.md5(doi.encode('utf-8')).hexdigest().lower()

    # 提取目录层级：前2位作为一级目录，3-4位作为二级目录
    first_dir = hash_md5[0:2].upper()
    second_dir = hash_md5[2:4].upper()

    return first_dir + "/" + second_dir


def get_doi_path(doi):
    # 目前使用
    dir_path = doi_to_dir(doi)
    file_name = base_lngid.getDoiid(doi) + ".pdf"
    return dir_path + "/" + file_name



import re

import unicodedata


def is_ascii_alnum(char: str) -> bool:
    #  精准判断是否为英文或数字字符（ASCII 范围）
    # char.isalnum() 字母或数字（Unicode）包含中文等非英文字符
    return char.isascii() and char.isalnum()


def is_all_english_chars(s):
    return bool(re.match(r'^[A-Za-z]+$', s))


def contains_chinese_chars(s):
    return bool(re.search(r'[\u3400-\u9fff]', s))


def is_empty(value):
    """
    判断一个值是否为空。

    支持的类型：
    - None
    - 空字符串（去除空白后）
    - pandas 的 NaN
    - 其他可迭代类型（如列表、字典等）的长度为 0
    - 其他情况返回 False
    """
    # 如果是 None，直接返回 True
    if value is None:
        return True

    # 尝试处理 pandas 的 NaN
    try:
        import pandas as pd
        if pd.isna(value):
            return True
    except:
        pass  # 如果没有安装 pandas，跳过

    # 如果是字符串，检查去除空白后是否为空
    if isinstance(value, str):
        return value.strip() == ""

    # 处理其他可迭代类型（如列表、字典等）
    if hasattr(value, "__len__"):
        return len(value) == 0

    # 默认情况下，非 None、非空类型返回 False
    return False


class InvalidCharLengthError(Exception):
    """自定义异常类，用于处理输入字符长度不为 1 的情况"""
    pass


def is_single_cjk_char(char):
    """
    判断单个字符是否为中日韩字符
    :param char: 要判断的单个字符
    :return: 如果是中日韩字符返回 True，否则返回 False
    """
    # 检查输入字符的长度
    if len(char) != 1:
        raise InvalidCharLengthError("输入的字符串长度必须为 1，请提供单个字符进行判断。")
    code_point = ord(char)
    # 中日韩统一表意文字
    ranges = [
        (0x4E00, 0x9FFF),  # CJK 统一表意符号
        (0x3400, 0x4DBF),  # CJK 统一表意符号扩展 A
        (0x20000, 0x2A6DF),  # CJK 统一表意符号扩展 B
        (0x2A700, 0x2B73F),  # CJK 统一表意符号扩展 C
        (0x2B740, 0x2B81F),  # CJK 统一表意符号扩展 D
        (0x2B820, 0x2CEAF),  # CJK 统一表意符号扩展 E
        (0x2CEB0, 0x2EBEF),  # CJK 统一表意符号扩展 F
        (0x30000, 0x3134F),  # CJK 统一表意符号扩展 G
        (0x31350, 0x323AF),  # CJK 统一表意符号扩展 H
        (0x3300, 0x33FF),  # CJK 兼容符号
        (0xFE30, 0xFE4F),  # CJK 兼容形式
        (0xF900, 0xFAFF),  # CJK 兼容表意符号
        (0x2F800, 0x2FA1F),  # CJK 兼容表意符号补充
        (0x3105, 0x3129),  # 注音字母
        (0x31A0, 0x31BF),  # 注音字母扩展
        (0x3040, 0x309F),  # 平假名
        (0x30A0, 0x30FF),  # 片假名
        (0x31F0, 0x31FF),  # 片假名扩展
        (0xAC00, 0xD7AF),  # 韩文音节
        (0x1100, 0x11FF),  # 韩文字母
        (0xA960, 0xA97F),  # 韩文字母扩展 A
        (0xD7B0, 0xD7FF),  # 韩文字母扩展 B
    ]
    for start, end in ranges:
        if start <= code_point <= end:
            return True
    return False


def is_all_symbols(text):
    # 是否全是符号
    # 如果字符串为空，返回 False
    if not text:
        return False

    # 检查每个字符是否属于符号类别
    return all(unicodedata.category(char).startswith(('P', 'S')) for char in text)


def is_whole_word_en(sub_str: str, long_str: str) -> bool:
    """
    判断 sub_str 是否作为 long_str 中的一个完整英文单词（不被其他单词嵌套）。

    参数:
        sub_str: 要搜索的英文子串
        long_str: 被搜索的字符串

    返回:
        True 表示 sub_str 是一个完整单词；False 表示是部分单词或不匹配。
    """
    # 用于 忽略大小写 进行匹配
    regex_pattern = re.compile(r"[^a-z0-9]", re.IGNORECASE)  # 用于判断非字母数字字符

    if not sub_str or not long_str:
        return False

    # 检查整段是否完全等于 sub_str
    if long_str == sub_str:
        return True

    # 遍历所有 sub_str 的出现位置
    index = 0
    while index < len(long_str):
        # 从字符串 long_str 的第 index 个位置开始，查找子串 sub_str 第一次出现的位置，并把它赋值给 index。
        index = long_str.find(sub_str, index)
        if index == -1:
            break

        # 检查 sub_str 前一个字符（如果有）是否为非字母数字
        if index == 0:
            is_start = True
        else:
            is_start = bool(regex_pattern.match(long_str[index - 1]))

        # 检查 sub_str 后一个字符（如果有）是否为非字母数字
        end_index = index + len(sub_str)
        if end_index == len(long_str):
            is_end = True
        else:
            is_end = bool(regex_pattern.match(long_str[end_index]))

        if is_start and is_end:
            return True

        # 移动索引继续查找
        index += 1

    return False


def is_whole_word(sub_str: str, long_str: str) -> bool:
    """
    判断 sub_str 是否为 long_str 中的一个完整词（适配中英文）。
    中文采用“包含”判断，英文采用完整词匹配。

    参数:
        sub_str: 要搜索的子串（中英文均可）
        long_str: 被搜索的字符串

    返回:
        True 表示 sub_str 是一个完整词；False 否则。
    """
    if contains_chinese_chars(sub_str):
        # 子字符串完全包含在长字符串
        is_contain = sub_str in long_str
        # 是否是字母数字
        if is_ascii_alnum(sub_str[0]) or is_ascii_alnum(sub_str[-1]):
            # 表示中英文混合 看是否是截断单词即可
            return is_whole_word_en(sub_str, long_str)
        else:
            # 中文子串只要被包含即可视为“完整词”
            return is_contain
    else:
        # 英文使用完整单词判断逻辑
        return is_whole_word_en(sub_str, long_str)



from abc import ABC, abstractmethod
from typing import List, Generator


class BaseFileReader(ABC):

    def __init__(self, batch_size: int = 10000):
        self.batch_size = batch_size
        self.read_model = 1

    @abstractmethod
    def list_files(self, path: str) -> List[str]:
        """列出路径下所有目标文件"""
        pass

    @abstractmethod
    def count_lines(self, file_path: str) -> int:
        """统计文件行数"""
        pass

    @abstractmethod
    def read_lines(self, file_path: str) -> Generator[List[str], None, None]:
        """读取文件内容，返回批量数据"""
        pass

    @abstractmethod
    def read_all(self, file_path: str) -> List[List[str]]:
        """读取整个文件，默认按1000行分批"""
        return [line for line in self.read_lines(file_path)]

    def read_select(self, file_path: str) -> Generator[List[str], None, None]:
        if self.read_model == 1:
            for batch_data in self.read_lines(file_path):
                yield batch_data
        elif self.read_model == 2:
            for batch_data in self.read_all(file_path):
                yield batch_data
        else:
            raise Exception("模式选择错误")


class BaseFileWriter(ABC):

    def __init__(self, file_path: str, compress: bool = True, overwrite: bool = True, encoding: str = "utf-8"):
        self.file_path = file_path
        self.compress = compress
        self.encoding = encoding
        self.overwrite = overwrite

    @abstractmethod
    def write_lines(self, lines: List[str], file_path: str):
        """写入多行文本到文件，支持压缩"""
        pass



import gzip
import io
import json
from io import BytesIO
from pathlib import Path
from typing import List, Generator

import pandas as pd
from hdfs import InsecureClient

from re_common.v2.baselibrary.tools.data_processer.base import BaseFileReader


class HDFSFileReader(BaseFileReader):
    def __init__(self, batch_size: int = 1000, hdfs_url: str = "http://VIP-DC-MASTER-2:9870", hdfs_user: str = "root"):
        super().__init__(batch_size)
        self.client = InsecureClient(hdfs_url, user=hdfs_user)

    def list_files(self, path: str) -> List[str]:
        return [f"{path}/{f[0]}" for f in self.client.list(path, status=True) if f[0] != '_SUCCESS']

    def count_lines(self, file_path: str) -> int:
        with self.client.read(file_path) as f:
            return sum(1 for _ in f)

    def read_lines(self, file_path: str) -> Generator[List[str], None, None]:
        # 批量读取后 处理 缺点 连接可能会断
        with self.client.read(file_path) as f:
            while True:
                batch = []
                for _ in range(self.batch_size):
                    try:
                        line = next(f)
                        line = line.decode('utf-8')
                        if line.strip():
                            batch.append(line.strip())
                    except StopIteration:
                        break
                if not batch:
                    break
                yield batch

    def read_all(self, file_path: str) -> List[List[str]]:
        # 一次读取返回所有后批量处理缺点 内存占用
        with self.client.read(file_path) as f:
            lines = [line.decode('utf-8').strip() for line in f if line.decode('utf-8').strip()]
            return [lines[i: i + self.batch_size] for i in range(0, len(lines), self.batch_size)]


class HDFSGZFileReader(BaseFileReader):
    def __init__(self, batch_size: int = 1000, hdfs_url: str = "http://VIP-DC-MASTER-2:9870", hdfs_user: str = "root"):
        super().__init__(batch_size)
        self.hdfs_url = hdfs_url
        self.hdfs_user = hdfs_user
        self.client = None

    def _init_client(self):
        if self.client is None:
            self.client = InsecureClient(self.hdfs_url, user=self.hdfs_user)
        return self

    def list_files(self, path: str) -> List[str]:
        self._init_client()
        return [f"{path}/{f[0]}" for f in self.client.list(path, status=True) if f[0].endswith(".gz")]

    def count_lines(self, file_path: str) -> int:
        self._init_client()
        with self.client.read(file_path) as f:
            with gzip.GzipFile(fileobj=f) as gz:
                return sum(1 for _ in gz)

    def read_lines(self, file_path: str) -> Generator[List[str], None, None]:
        self._init_client()
        # 批量读取后 处理 缺点 连接可能会断
        with self.client.read(file_path) as f:
            with gzip.GzipFile(fileobj=f) as gz:
                while True:
                    batch = []
                    for _ in range(self.batch_size):
                        try:
                            line = next(gz)
                            if line.strip():
                                batch.append(line.decode("utf-8"))
                        except StopIteration:
                            break
                    if not batch:
                        break
                    yield batch

    def read_all(self, file_path: str) -> List[List[str]]:
        self._init_client()
        # 一次读取返回所有后批量处理缺点 内存占用
        with self.client.read(file_path) as reader:
            compressed_data = reader.read()
            with gzip.GzipFile(fileobj=BytesIO(compressed_data)) as gz_file:
                content = gz_file.read().decode("utf-8")
                lines = [i for i in content.split("\n") if i.strip()]
                return [lines[i: i + self.batch_size] for i in range(0, len(lines), self.batch_size)]


class HDFSParquetFileReader(BaseFileReader):
    def __init__(self, batch_size: int = 1000, hdfs_url: str = "http://VIP-DC-MASTER-2:9870", hdfs_user: str = "root"):
        super().__init__(batch_size)
        self.client = InsecureClient(hdfs_url, user=hdfs_user)

    def list_files(self, path: str) -> List[str]:
        return [f"{path}/{f[0]}" for f in self.client.list(path, status=True) if f[0].endswith(".parquet")]

    def count_lines(self, file_path: str) -> int:
        with self.client.read(file_path) as f:
            data = f.read()
            df = pd.read_parquet(io.BytesIO(data))
        count = len(df)
        return count

    def read_lines(self, file_path: str) -> Generator[List[str], None, None]:
        # 批量读取后 处理 缺点 连接可能会断
        with self.client.read(file_path) as f:
            data = f.read()
            df = pd.read_parquet(io.BytesIO(data))
            records = [json.dumps(row, ensure_ascii=False) for row in df.to_dict(orient='records')]
            for i in range(0, len(records), self.batch_size):
                yield records[i: i + self.batch_size]

    def read_all(self, file_path: str) -> List[List[str]]:
        # 一次读取返回所有后批量处理缺点 内存占用
        with self.client.read(file_path) as f:
            data = f.read()
            df = pd.read_parquet(io.BytesIO(data))
            records = [json.dumps(row, ensure_ascii=False) for row in df.to_dict(orient='records')]
            return [records[i: i + self.batch_size] for i in range(0, len(records), self.batch_size)]


class LocalGZFileReader(BaseFileReader):
    def list_files(self, path: str) -> List[str]:
        return [str(p) for p in Path(path).rglob("*.gz")]

    def count_lines(self, file_path: str) -> int:
        with gzip.open(file_path, 'rt', encoding='utf-8') as f:
            return sum(1 for _ in f)

    def read_lines(self, file_path: str) -> Generator[List[str], None, None]:
        with gzip.open(file_path, 'rt', encoding='utf-8') as f:
            while True:
                batch = []
                for _ in range(self.batch_size):
                    line = f.readline()
                    if not line:
                        break
                    if line.strip():
                        batch.append(line.strip())
                if not batch:
                    break
                yield batch

    def read_all(self, file_path: str) -> List[List[str]]:
        with gzip.open(file_path, 'rt', encoding='utf-8') as f:
            lines = [line.strip() for line in f if line.strip()]
            return [lines[i: i + self.batch_size] for i in range(0, len(lines), self.batch_size)]


class LocalFileReader(BaseFileReader):
    def list_files(self, path: str) -> List[str]:
        return [str(p) for p in Path(path).rglob("*") if p.is_file()]

    def count_lines(self, file_path: str) -> int:
        with open(file_path, 'r', encoding='utf-8') as f:
            return sum(1 for _ in f)

    def read_lines(self, file_path: str) -> Generator[List[str], None, None]:
        with open(file_path, 'r', encoding='utf-8') as f:
            while True:
                batch = []
                for _ in range(self.batch_size):
                    line = f.readline()
                    if not line:
                        break
                    if line.strip():
                        batch.append(line.strip())
                if not batch:
                    break
                yield batch

    def read_all(self, file_path: str) -> List[List[str]]:
        with open(file_path, 'r', encoding='utf-8') as f:
            lines = [line.strip() for line in f if line.strip()]
            return [lines[i: i + self.batch_size] for i in range(0, len(lines), self.batch_size)]



import pymssql


class BaseMsSql(object):

    def __init__(self, host, user, pwd, db, charset, **kwargs):
        """
        as_dict  作为字典返回
        :param host:
        :param user:
        :param pwd:
        :param db:
        :param kwargs:
        """
        self.host = host
        self.user = user
        self.pwd = pwd
        self.db = db
        self.charset = charset
        self.kwargs = kwargs

    def conn(self):
        if not self.db:
            raise (NameError, "没有设置数据库信息")
        self.conn = pymssql.connect(host=self.host,
                                    user=self.user,
                                    password=self.pwd,
                                    database=self.db,
                                    charset=self.charset,
                                    **self.kwargs)
        self.cur = self.conn.cursor()
        if not self.cur:
            raise (NameError, "连接数据库失败")
        else:
            return self.cur

    def exec_select_query(self, sql):
        """
        执行查询语句
        返回的是一个包含tuple的list，list的元素是记录行，
        tuple的元素是每行记录的字段
        """
        try:
            self.cur.execute(sql)
            return self.cur
        except:
            self.close()

    def fetchone(self):
        """
        获取查询结果的下一行，返回一个元组，或者如果as_dict为True的话
        返回一个字典。如果没有更多可获取的数据了，返回None。
        如果前一个对execute*() 的调用没有产生任何结果集或者还没有提交调用，
        则抛出OperationalError
        :return:
        """
        return self.cur.fetchone()

    def fetchmany(self, size=None):
        """
        获取下一批查询结果，返回一个元组的列表，或者如果as_dict为True的话返回
        一个字典。如果没有更多可获取的数据了，返回一个空列表。
        你可以使用size参数调整之后每一批获取的行数，这个值会一直保留使用。
        如果前一个对execute*() 的调用没有产生任何结果集或者还没有提交调用，
        则抛出OperationalError
        :param size:
        :return:
        """
        return self.cur.fetchmany(size=size)

    def fetchall(self):
        """
        获取查询结果的所有剩余行，返回一个元组的列表，或者如果as_dict为True
        的话返回一个字典。如果没有更多可获取的数据了，返回一个空列表。
        如果前一个对execute*() 的调用没有产生任何结果集或者还没有提交调用，
        则抛出OperationalError
        :return:
        """
        return self.cur.fetchall()

    def exec_non_query(self, sql):
        """
        执行非查询语句
        """
        try:
            self.cur.execute(sql)
            self.conn.commit()
            return True
        except:
            self.close()
            return False

    def close(self):
        self.cur.close()
        self.conn.close()

    def __del__(self):
        self.close()



from boto3.session import Session


class BaseBoto3(object):

    def __init__(self, aws_access_key_id="", aws_secret_access_key="", endpoint_url=""):
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.endpoint_url = endpoint_url
        self.session = None
        self.client = None
        if self.aws_access_key_id and self.aws_secret_access_key and self.endpoint_url:
            self.conn_session()
            self.get_client()

    def set_key(self, aws_access_key_id, aws_secret_access_key, endpoint_url):
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.endpoint_url = endpoint_url
        return self

    def conn_session(self):
        assert self.aws_access_key_id not in (None, '')
        assert self.aws_secret_access_key not in (None, '')
        self.session = Session(aws_access_key_id=self.aws_access_key_id,
                               aws_secret_access_key=self.aws_secret_access_key)
        return self.session

    def get_client(self):
        assert self.session is not None
        self.client = self.session.client('s3', endpoint_url=self.endpoint_url)
        return self

    def get_all_buckets(self):
        """
        获取所有的桶信息
        :return:
        """
        return self.client.list_buckets()

    def create_buckets(self, buckets_name):
        """
         如果get_client 使用 client 返回
        {'ResponseMetadata': {'RequestId': '16BC90EED4A433C4', 'HostId': '', 'HTTPStatusCode': 200, 'HTTPHeaders': {'accept-ranges': 'bytes', 'content-length': '0', 'content-security-policy': 'block-all-mixed-content', 'location': '/create1', 'server': 'MinIO', 'strict-transport-security': 'max-age=31536000; includeSubDomains', 'vary': 'Origin, Accept-Encoding', 'x-amz-request-id': '16BC90EED4A433C4', 'x-content-type-options': 'nosniff', 'x-xss-protection': '1; mode=block', 'date': 'Wed, 01 Dec 2021 07:28:39 GMT'}, 'RetryAttempts': 0}, 'Location': '/create1'}
        """
        assert buckets_name.find("_") == -1, "新建一个bucket桶(bucket name 中不能有_下划线)"
        # 新建一个bucket桶(bucket name 中不能有_下划线)
        return self.client.create_bucket(Bucket=buckets_name)

    def delete_buckets(self, bucket_name):
        """
        删除桶 删除bucket(只能删除空的bucket)
        :return:
        """
        response = self.client.delete_bucket(Bucket=bucket_name)
        return response

    def get_bucket(self, bucket_name):
        raise Exception("无实现方法")

    def get_all_objs(self, bucket_name, prefix=None, continuation_token=None):
        """

        continuation_token: 如果超过1000 需要传第一次获取结果中的 continuation_token

        response 的结构
        {'ResponseMetadata': {'RequestId': '1818F447C1E7BA3B', 'HostId': '', 'HTTPStatusCode': 200,
        'HTTPHeaders': {'accept-ranges': 'bytes', 'content-length': '3182', 'content-security-policy': 'block-all-mixed-content', 'content-type': 'application/xml',
        'server': 'MinIO', 'strict-transport-security': 'max-age=31536000; includeSubDomains', 'vary': 'Origin, Accept-Encoding', 'x-amz-request-id': '1818F447C1E7BA3B',
         'x-content-type-options': 'nosniff', 'x-xss-protection': '1; mode=block', 'date': 'Thu, 09 Jan 2025 07:04:05 GMT'}, 'RetryAttempts': 0},
         'IsTruncated': False, 'Contents':
         [
         {'Key': 'zt_file/zt类型样例数据/11_part-00000.gz', 'LastModified': datetime.datetime(2024, 4, 28, 2, 56, 59, 716000, tzinfo=tzutc()), 'ETag': '"e0d635f171bce6a67ad72265e5f9137d-2"',
          'Size': 18164139, 'StorageClass': 'STANDARD', 'Owner': {'DisplayName': 'minio', 'ID': '02d6176db174dc93cb1b899f7c6078f08654445fe8cf1b6ce98d8855f66bdbf4'}},
        {'Key': 'zt_file/zt类型样例数据/12_part-00000.gz', 'LastModified': datetime.datetime(2024, 4, 28, 2, 56, 57, 70000, tzinfo=tzutc()), 'ETag': '"f238fe9973a2bc0d3e1562c2938ce897-9"',
        'Size': 93710911, 'StorageClass': 'STANDARD', 'Owner': {'DisplayName': 'minio', 'ID': '02d6176db174dc93cb1b899f7c6078f08654445fe8cf1b6ce98d8855f66bdbf4'}},
         ],
         'Name': 'crawl.dc.cqvip.com', 'Prefix': 'zt_file/zt类型样例数据', 'Delimiter': '',
         'MaxKeys': 1000, 'EncodingType': 'url', 'KeyCount': 7}

        """
        if continuation_token:
            # 获取桶中以特定前缀开头的所有对象
            response = self.client.list_objects_v2(Bucket=bucket_name,
                                                   Prefix=prefix,
                                                   ContinuationToken=continuation_token)
        else:
            # 获取桶中以特定前缀开头的所有对象
            response = self.client.list_objects_v2(Bucket=bucket_name,
                                                   Prefix=prefix)
        object_list = []
        # 检查是否有对象存在
        if 'Contents' in response:
            object_list = [obj['Key'] for obj in response['Contents']]

        continuation_token = None
        # 检查是否有更多对象
        if response.get('IsTruncated'):  # 如果返回结果被截断，说明有更多对象
            continuation_token = response.get('NextContinuationToken')

        return object_list, continuation_token

    def list_prefixes(self, bucket_name, prefix=None, Delimiter="/", continuation_token=None):
        """
        获取目录下一层的目录
        prefix: 注意 这个要以 Delimiter 结尾 比如 Delimiter="/" 那么 prefix="a/"
        continuation_token: 如果超过1000 需要传第一次获取结果中的 continuation_token
        return:  ['a/b/', 'a/c/'] 注意 反回的 结果带有prefix 只能返回目录 不能返回文件
        """
        if continuation_token:
            # 获取桶中以特定前缀开头的所有对象
            response = self.client.list_objects_v2(Bucket=bucket_name,
                                                   Prefix=prefix,
                                                   Delimiter=Delimiter,  # 使用斜杠分隔符模拟目录结构
                                                   ContinuationToken=continuation_token)
        else:
            # 获取桶中以特定前缀开头的所有对象
            response = self.client.list_objects_v2(Bucket=bucket_name,
                                                   Delimiter=Delimiter,  # 使用斜杠分隔符模拟目录结构
                                                   Prefix=prefix)
        object_list = []
        # 检查是否有对象存在
        if 'Contents' in response:
            object_list = [obj['Key'] for obj in response['Contents']]

        Prefix_list = []
        # 检查是否有目录存在
        if 'CommonPrefixes' in response:
            Prefix_list = [obj['Prefix'] for obj in response['CommonPrefixes']]

        continuation_token = None
        # 检查是否有更多对象
        if response.get('IsTruncated'):  # 如果返回结果被截断，说明有更多对象
            continuation_token = response.get('NextContinuationToken')

        return object_list, Prefix_list, continuation_token

    def get_object_value(self, bucket_name, file_key, encoding='utf-8'):
        """
        读取文本数据
        Returns:
        """
        obj = self.client.get_object(Bucket=bucket_name, Key=file_key)
        body = obj['Body'].read().decode(encoding)
        return body

    def put_object(self, bucket_name, key, body):
        """
        直接写内容到文件
        Args:
            bucket_name:
            key:
            body: 需要 编码 .encode('utf-8')

        Returns:
        """
        self.client.put_object(Bucket=bucket_name,
                               Key=key,
                               Body=body)

    def download_file(self, bucket_name, key, local_file):
        """
        return: None
        """
        result = self.client.download_file(bucket_name, key, local_file)
        return result

    def upload_file(self, bucket_name, key, local_file):
        """
        # key 桶中的位置 test1/test.pdf
        :param local_file:  本地文件路径
        :param bucket_name: 桶名
        :param key: 远程文件路径
        :return:
        """
        self.client.upload_file(local_file, bucket_name, key)

    def download_fileobj(self, bucket_name, key, fileobj):
        """
        return: None
        """
        result = self.client.download_fileobj(bucket_name, key, fileobj)
        return result

    def upload_fileobj(self, bucket_name, key, fileobj):
        # fileobj 字节流
        self.client.upload_fileobj(fileobj, bucket_name, key)

    def check_exist_or_file_info(self, bucket_name, key):
        """
        检查文件是否存在且能获取文件info
        {'ResponseMetadata': {'RequestId': '17E6A65A2B299D3B', 'HostId': '', 'HTTPStatusCode': 200, 'HTTPHeaders':
         {'accept-ranges': 'bytes', 'content-length': '117', 'content-security-policy': 'block-all-mixed-content', 'content-type': 'binary/octet-stream',
         'etag': '"2237a934f176003e41abf3d733291079"', 'last-modified': 'Thu, 25 Jul 2024 05:49:43 GMT', 'server': 'MinIO',
         'strict-transport-security': 'max-age=31536000; includeSubDomains', 'vary': 'Origin, Accept-Encoding', 'x-amz-request-id': '17E6A65A2B299D3B',
          'x-content-type-options': 'nosniff', 'x-xss-protection': '1; mode=block', 'date': 'Mon, 29 Jul 2024 09:53:33 GMT'}, 'RetryAttempts': 0},
          'AcceptRanges': 'bytes', 'LastModified': datetime.datetime(2024, 7, 25, 5, 49, 43, tzinfo=tzutc()), 'ContentLength': 117, 'ETag': '"2237a934f176003e41abf3d733291079"',
           'ContentType': 'binary/octet-stream', 'Metadata': {}}
        """
        try:
            obj_info = self.client.head_object(
                Bucket=bucket_name,
                Key=key
            )
            return obj_info
        except:
            return None

    def get_prefix_count(self, bucket_name, obj_count, prefix, continuation_token=None):
        """
        统计 某个目录的文件数据量，由于需要每个目录获取一次 性能很慢
        """
        for index in range(10000):
            obj_list, dir_list, token = self.list_prefixes(bucket_name=bucket_name,
                                                           prefix=prefix,
                                                           continuation_token=continuation_token)

            obj_count = obj_count + len(obj_list)
            for dir_sub in dir_list:
                obj_count = self.get_prefix_count(bucket_name, obj_count, dir_sub)

            if token:
                continuation_token = token
            else:
                break

        if index > 10000 - 5:
            raise Exception("循环耗尽，请检查逻辑正确性")

        return obj_count



from re_common.baselibrary.baseabs import BaseAbs


class SqlFactory(BaseAbs):
    """
    sql 工厂类
    """
    @staticmethod
    def mysql_factory(type='mysql'):
        if type == 'mysql':
            """
            mysql类 返回一个mysql的对象
            """
            from re_common.baselibrary.database.mysql import Mysql
            return Mysql()
        assert 0, "err sql type please check: %s" % type

    @staticmethod
    def sqlite_factory(type='sqlite3'):
        if type == 'sqlite3':
            """
             sqlite3类 返回一个sqlite3的对象
             """
            from re_common.baselibrary.database.msqlite3 import Sqlite3
            return Sqlite3()
        assert 0, "err sqllite type please check: %s" % type



import json
###########################################
# 同项目调用基础包
import os
import sys
import time

import requests

filepath = os.path.abspath(__file__)
pathlist = filepath.split(os.sep)
pathlist = pathlist[:-4]
TopPath = os.sep.join(pathlist)
sys.path.insert(0, TopPath)
print(TopPath)
############################################

from re_common.baselibrary.utils.basedir import BaseDir
from re_common.baselibrary.utils.basefile import BaseFile
from re_common.baselibrary.utils.baserequest import BaseRequest
from re_common.facade.lazy_import import get_streamlogger
from re_common.facade.mysqlfacade import MysqlUtiles
from re_common.baselibrary.utils.basetime import BaseTime


class Kproxy(object):
    def __init__(self):
        self.cur_path = BaseDir.get_file_dir_absolute(__file__)
        self.configfile = BaseFile.get_new_path(self.cur_path, "db.ini")
        self.logger = get_streamlogger()
        self.mysqlutils = MysqlUtiles(self.configfile, "allproxy", self.logger)
        self.bsrequest = BaseRequest()
        self.starttime = time.time()
        self.starttime_val = time.time()

    def get_taiyang_proxy(self, num=6):
        """
        获取太阳代理 每分钟3个
        :param num:
        :return:
        """
        self.starttime = time.time()
        #  url = "http://http.tiqu.qingjuhe.cn/getip?num={}&type=2&pack=59105&port=1&ts=1&lb=1&pb=45&regions=".format(num)
        url = "http://http.tiqu.alibabaapi.com/getip?num={}&type=2&pack=59105&port=1&ts=1&lb=1&pb=45&regions=".format(num)
        BoolResult, errString, r = self.bsrequest.base_request(url,
                                                               timeout=30
                                                               )
        if BoolResult:
            dicts = json.loads(r.text)
            for item in dicts["data"]:
                proxy = item["ip"] + ":" + item["port"]
                sources = "taiyang"
                expire_time = item["expire_time"]
                sql = "insert into proxyall_v1 (proxy,sources,expire_time) values ('%s','%s','%s') on DUPLICATE key update stat=1,expire_time='%s'" % (
                    proxy, sources, expire_time, expire_time)
                self.mysqlutils.ExeSqlToDB(sql)
        else:
            self.logger.error("获取失败")

    def get_taiyang_num(self):
        """
        获取太阳代理 每分钟3个
        :param num:
        :return:
        """
        url = "http://ty-http-d.hamir.net/index/index/get_my_package_balance?neek=521821&appkey=1fcba6de94f71561ba3007f4c24ca0b1&ac=59105"
        BoolResult, errString, r = self.bsrequest.base_request(url,
                                                               timeout=30
                                                               )
        if BoolResult:
            dicts = json.loads(r.text)
            return dicts["data"]["package_balance"]
        else:
            self.logger.error("获取失败")

    def val(self, proxy, sources):
        # 请求地址
        targetUrl = "https://www.baidu.com"
        proxies = {
            "http": "http://%s" % proxy,
            "https": "http://%s" % proxy
        }
        resp = requests.get(targetUrl, proxies=proxies, timeout=5)
        if resp.status_code == 200:
            print(resp.status_code)
            return True
        else:
            sql = "update proxyall_v1 set stat=0 where proxy='%s' and sources='%s';" % (proxy, sources)
            self.mysqlutils.ExeSqlToDB(sql)
            return False

    def val_all(self):
        self.starttime_val = time.time()
        sql = "select proxy,sources from proxyall_v1 where stat=1 ORDER BY `update_time`"
        bools, rows = self.mysqlutils.SelectFromDB(sql)
        for row in rows:
            try:
                self.val(row[0], row[1])
            except:
                sql = "update proxyall_v1 set stat=0 where proxy='%s' and sources='%s';" % (row[0], row[1])
                self.mysqlutils.ExeSqlToDB(sql)

    def run(self):
        sleep_time = 0
        while True:
            num = 10
            if sleep_time < 0:
                print("time sleep {}".format(str(100 + sleep_time)))
                if 100 + sleep_time > 0:
                    time.sleep(100 + sleep_time)
                num = num + 10
            nowtime = BaseTime().get_beijin_date_strins(format="%H%M%S")
            print(nowtime)
            if "133700" <= nowtime <= "134700":
                num = Kproxy().get_taiyang_num()
            start_time = time.time()
            self.get_taiyang_proxy(num=num)
            self.val_all()
            use_time = int(time.time() - start_time)
            sleep_time = 100 - use_time
            print("time sleep {}".format(str(sleep_time)))
            if sleep_time >= 3:
                time.sleep(sleep_time)


if __name__ == "__main__":
    Kproxy().run()



import json

###########################################
# 同项目调用基础包
import os
import sys
import time
import traceback

filepath = os.path.abspath(__file__)
pathlist = filepath.split(os.sep)
pathlist = pathlist[:-4]
TopPath = os.sep.join(pathlist)
sys.path.insert(0, TopPath)
print(TopPath)
############################################

from re_common.baselibrary.utils.baserequest import BaseRequest
from re_common.facade.mysqlfacade import MysqlUtiles
from re_common.baselibrary.utils.basedir import BaseDir
from re_common.baselibrary.utils.basefile import BaseFile
from re_common.facade.lazy_import import get_streamlogger


class Kproxy(object):
    def __init__(self):
        self.cur_path = BaseDir.get_file_dir_absolute(__file__)
        self.configfile = BaseFile.get_new_path(self.cur_path, "db.ini")
        self.logger = get_streamlogger()
        self.mysqlutils = MysqlUtiles(self.configfile, "dbkuaidaili", self.logger)
        self.url = "https://dps.kdlapi.com/api/getdps/?orderid=990171566857288&num=15&pt=1&format=json&sep=1"
        self.bsrequest = BaseRequest()
        self.starttime = time.time()
        self.starttime_val = time.time()

    def get_proxy(self):
        self.starttime = time.time()
        BoolResult, errString, r = self.bsrequest.base_request(self.url,
                                                               timeout=30
                                                               )
        if BoolResult:
            dicts = json.loads(r.text)
            for proxy in dicts["data"]["proxy_list"]:
                sql = "insert into kuaidailiproxy (proxy) values ('%s') on DUPLICATE key update stat=1" % proxy
                self.mysqlutils.ExeSqlToDB(sql)
        else:
            self.logger.error("获取失败")

    def val(self, proxy):
        self.starttime_val = time.time()
        url = f"https://dps.kdlapi.com/api/getdpsvalidtime?orderid=990171566857288&signature=wm4vq53pwrat1vye458elwyxyh9awzqj&proxy={proxy}"
        BoolResult, errString, r = self.bsrequest.base_request(url,
                                                               timeout=30
                                                               )
        if BoolResult:
            dicts = json.loads(r.text)
            if dicts["data"][proxy] > 0:
                sql = "update kuaidailiproxy set val_stat=0 where proxy='%s'" % proxy
                self.mysqlutils.ExeSqlToDB(sql)
            else:
                sql = "update kuaidailiproxy set val_stat=0,stat=0 where proxy='%s'" % proxy
                self.mysqlutils.ExeSqlToDB(sql)
        else:
            self.logger.error("获取失败")

    def val_all(self):
        self.starttime_val = time.time()
        sql = "select proxy from kuaidailiproxy where stat=1"
        bools, rows = self.mysqlutils.SelectFromDB(sql)
        for row in rows:
            try:
                self.val(row[0])
            except:
                traceback.print_exc()

    def run(self):
        self.get_proxy()
        while True:
            if int(time.time() - self.starttime) > 20:
                self.get_proxy()
            if int(time.time() - self.starttime_val) > 30:
                self.val_all()
            time.sleep(5)


if __name__ == "__main__":
    Kproxy().run()



import pydash
from pydash import without, py_, reject, power


class BasePyDash(object):

    @classmethod
    def find_index(cls, lists, callback):
        """
        lists = [
            {'name': 'Michelangelo', 'active': False},
            {'name': 'Donatello', 'active': False},
            {'name': 'Leonardo', 'active': True}
        ]
        通过callback 查找对应数据的索引,找的的第一个会被返回，
        索引从0开始，找不到返回-1
        :param lists:
        :param callback: callback的结果一般是个bool值
        # 第一种形式
        callback = lambda item: item['name'] == 'Leonardo'
        callback = lambda item, index: index == 3
        callback = lambda item, index, obj: obj[index]['active']
        # 浅属性风格
        在内部pydash.utilities.prop()用于创建回调
        callback=['active'] or 'active' or ['active',True]
        # deep 属性样式
        callback =  'location.city'
        :return: 索引或者-1
        """

        return pydash.find_index(lists, callback)

    @classmethod
    def map_(cls, lists, callback):
        """
        和 find_index 一样 只是返回的是数据而不是索引，且返回找到的全部
        没有找到的一项会返回None
        :param lists:
        :param callback:
        如果是给的字典map_(users, {'location': {'city': 'Florence'}}会返回true False的列表
        :return:
        """
        return pydash.map_(lists, callback)

    @classmethod
    def find_last_index(cls, lists, callback):
        """
        查找最后一个
        :param lists:
        :param callback:
        :return:
        """
        return pydash.find_last_index(lists, callback)

    @classmethod
    def get(cls, obj, path, default=None):
        """
        用. 获取多层字典的数据
        :return:
        """
        return pydash.get(obj, path, default)

    @classmethod
    def chain(cls, lists):
        """
        or
        from pydash import py_
        py_(lists)
        :param lists:
        :return:
        """
        pydash.chain(lists)
        return cls

    @classmethod
    def without(cls, array, *values):
        """
        without([1, 2, 3, 2, 4, 4], 2, 4)
        [1, 3]
        创建一个数组，删除所有已传递值的匹配项
        :return:
        """
        return without(array, *values)

    @classmethod
    def reject(cls,collection, predicate=None):
        """
        不接受某些条件
        reject([1, 2, 3, 4], lambda x: x >= 3)
        [1, 2]
        :param collection:
        :param predicate:
        :return:
        """
        return reject(collection, predicate=predicate)


    @classmethod
    def for_each(cls):
        """
        这里告诉你for-each 怎么用

        :return:
        """

        def echo(value): print(value)

        lazy = py_([1, 2, 3, 4]).for_each(echo)

        result = lazy.value()

    @classmethod
    def commit(cls):
        """
        提交链
        :return:
        """

        def echo(value): print(value)

        lazy = py_([1, 2, 3, 4]).for_each(echo)

        result = lazy.value()

        committed = lazy.commit()

        committed.value()

        # 相当于
        committed = py_(lazy.value())

    @classmethod
    def power(cls,x,n):
        """
        x的n次方
        :param x:
        :param n:
        :return:
        """


square_sum = py_().power(2).sum()
print(square_sum)



"""
一行文本按指定符号分割为多行文本
"""


class Split_2_lines(object):
    def __int__(self):
        pass

    def split_line(self, infilepath, sign, outfilepath=None):
        """
        :param infilepath: 输入文件位置
        :param sign: 分割符号
        :param outfilepath: 输出文件位置
        :return:
        """
        with open(infilepath, 'r', encoding='utf-8')as f:
            for line in f:
                list_ = line.split(sign)[:-1]
                for num in list_:
                    new_line = num + sign + "\n"
                    with open(outfilepath, 'a', encoding='utf-8')as fw:
                        fw.write(new_line)
                        print(new_line)
        print("输出{}成功".format(outfilepath))



import sys
import traceback

import pymysql

from re_common.baselibrary import MLogger
from re_common.baselibrary.baseabs import BaseAbs
from re_common.baselibrary.database.mbuilder import MysqlBuilderAbstract
from re_common.baselibrary.database.moudle import SqlMoudle
from re_common.baselibrary.readconfig.ini_config import IniConfig
from re_common.baselibrary.readconfig.toml_config import TomlConfig


class MysqlBuilder(MysqlBuilderAbstract):
    def __init__(self, configfile, configname, keytransformdicts=None):
        # sql的moudle对象 mysql通过该对象对接参数
        self.sqlmoudle = SqlMoudle()
        # 获取配置文件对象
        self.ic = IniConfig(configfile).builder()
        # 配置文件位置
        self.configfile = configfile
        # 配置文件的sections
        self.configname = configname
        # 使用key转换器转换key(主要用于配置文件的key与我们默认读取时的key不一致)
        self.keytransformdicts = keytransformdicts
        """
        python2
            self.config.readfp(open(configfile, 'rb'))
        python3
            self.config.read_file(open(configfile, 'r'))
        """

    def build_port(self, port):
        self.sqlmoudle.port = int(self.ic.get_value(self.configname, port))
        return self

    def build_server_address(self, host):
        self.sqlmoudle.host = self.ic.get_value(self.configname, host)
        return self

    def build_password(self, passwd):
        self.sqlmoudle.passwd = self.ic.get_value(self.configname, passwd)
        return self

    def use_db(self, db):
        self.sqlmoudle.db = self.ic.get_value(self.configname, db)
        return self

    def build_username(self, user):
        self.sqlmoudle.user = self.ic.get_value(self.configname, user)
        return self

    def build_chart(self, chartset):
        self.sqlmoudle.charset = self.ic.get_value(self.configname, chartset)
        return self

    def build_cursorclass(self, cursornum):
        if cursornum == 1:
            self.sqlmoudle.cursorclass = pymysql.cursors.DictCursor
        else:
            # 默认保持不变
            pass

        return self

    # def build_cursorclass(self,cursorclass):
    #     self.sqlmoudle.cursorclass = self.ic.get_value(self.configname, cursorclass)

    def get_moudle(self):
        """
        返回数据库连接需要的对象
        :return:
        """
        return self.sqlmoudle

    def build_all(self):
        if not self.keytransformdicts:
            # 不传入时不转换
            self.keytransformdicts = {"port": "port", "host": "host", "passwd": "passwd", "user": "user", "db": "db",
                                      "chartset": "chartset"}
        self.build_port(self.keytransformdicts["port"])
        self.build_server_address(self.keytransformdicts["host"])
        self.build_password(self.keytransformdicts["passwd"])
        self.build_username(self.keytransformdicts["user"])
        self.use_db(self.keytransformdicts["db"])
        self.build_chart(self.keytransformdicts["chartset"])
        return self

    def get_tuples(self):
        return self.sqlmoudle.host, self.sqlmoudle.user, self.sqlmoudle.passwd, self.sqlmoudle.port


class MysqlBuilderForToml(MysqlBuilderAbstract):
    def __init__(self, configfile, sesc, keytransformdicts=None):
        # sql的moudle对象 mysql通过该对象对接参数
        self.sqlmoudle = SqlMoudle()
        # 获取配置文件对象
        self.dic = TomlConfig(configfile).read_file_remove_bom().get_dicts()
        # 配置文件位置
        self.configfile = configfile
        # 配置文件的sections
        self.sesc = sesc
        # 使用key转换器转换key(主要用于配置文件的key与我们默认读取时的key不一致)
        self.keytransformdicts = keytransformdicts
        """
        python2
            self.config.readfp(open(configfile, 'rb'))
        python3
            self.config.read_file(open(configfile, 'r'))
        """

    def build_port(self, port):
        self.sqlmoudle.port = int(self.dic[self.sesc][port])
        return self

    def build_server_address(self, host):
        self.sqlmoudle.host = self.dic[self.sesc][host]
        return self

    def build_password(self, passwd):
        self.sqlmoudle.passwd = self.dic[self.sesc][passwd]
        return self

    def use_db(self, db):
        self.sqlmoudle.db = self.dic[self.sesc][db]
        return self

    def build_username(self, user):
        self.sqlmoudle.user = self.dic[self.sesc][user]
        return self

    def build_chart(self, chartset):
        self.sqlmoudle.charset = self.dic[self.sesc][chartset]
        return self

    def build_cursorclass(self, cursornum):
        if cursornum == 1:
            self.sqlmoudle.cursorclass = pymysql.cursors.DictCursor
        else:
            # 默认保持不变
            pass

        return self

    def get_moudle(self):
        """
        返回数据库连接需要的对象
        :return:
        """
        return self.sqlmoudle

    def build_all(self):
        if not self.keytransformdicts:
            # 不传入时不转换
            self.keytransformdicts = {"port": "port", "host": "host", "passwd": "passwd", "user": "user", "db": "db",
                                      "chartset": "chartset"}
        self.build_port(self.keytransformdicts["port"])
        self.build_server_address(self.keytransformdicts["host"])
        self.build_password(self.keytransformdicts["passwd"])
        self.build_username(self.keytransformdicts["user"])
        self.use_db(self.keytransformdicts["db"])
        self.build_chart(self.keytransformdicts["chartset"])
        return self

    def get_tuples(self):
        return self.sqlmoudle.host, self.sqlmoudle.user, self.sqlmoudle.passwd, self.sqlmoudle.port


class MysqlBuilderForDicts(MysqlBuilderAbstract):
    def __init__(self, dicts, keytransformdicts=None):
        # sql的moudle对象 mysql通过该对象对接参数
        self.sqlmoudle = SqlMoudle()
        self.dicts = dicts
        # 使用key转换器转换key(主要用于配置文件的key与我们默认读取时的key不一致)
        self.keytransformdicts = keytransformdicts

    def build_port(self, port):
        self.sqlmoudle.port = int(self.dicts[port])
        return self

    def build_server_address(self, host):
        self.sqlmoudle.host = self.dicts[host]
        return self

    def build_password(self, passwd):
        self.sqlmoudle.passwd = self.dicts[passwd]
        return self

    def use_db(self, db):
        self.sqlmoudle.db = self.dicts[db]
        return self

    def build_username(self, user):
        self.sqlmoudle.user = self.dicts[user]
        return self

    def build_chart(self, chartset):
        self.sqlmoudle.charset = self.dicts[chartset]
        return self

    def build_cursorclass(self, cursornum):
        if cursornum == 1:
            self.sqlmoudle.cursorclass = pymysql.cursors.DictCursor
        else:
            # 默认保持不变
            pass

        return self

    def get_moudle(self):
        """
        返回数据库连接需要的对象
        :return:
        """
        return self.sqlmoudle

    def build_all(self):
        if not self.keytransformdicts:
            # 不传入时不转换
            self.keytransformdicts = {"port": "port", "host": "host", "passwd": "passwd", "user": "user", "db": "db",
                                      "chartset": "chartset"}
        self.build_port(self.keytransformdicts["port"])
        self.build_server_address(self.keytransformdicts["host"])
        self.build_password(self.keytransformdicts["passwd"])
        self.build_username(self.keytransformdicts["user"])
        self.use_db(self.keytransformdicts["db"])
        self.build_chart(self.keytransformdicts["chartset"])
        return self

    def get_tuples(self):
        return self.sqlmoudle.host, self.sqlmoudle.user, self.sqlmoudle.passwd, self.sqlmoudle.port


class MysqlUtiles(object):
    def __init__(self, cfgfilepath, sesc, logger=None, keytransformdicts=None, builder="MysqlBuilder", cursorsnum=0,
                 dicts=None):
        """

        :param cfgfilepath:
        :param sesc:
        :param logger:
        :param keytransformdicts:
        :param builder:
        :param cursors: 0表示默认，1 表示dicts 其他情况后面再添加
        """
        self._logger = logger
        # 通过工厂方法获取mysql对象
        self.mysql = BaseAbs.get_sql_factory().mysql_factory("mysql")
        # mysql 连接通过builder进行适配 这里的builder通过配置文件获取相关连接信息
        # 也可以通过继承 MysqlBuilderAbstract 后自己手动设置builer
        if builder == "MysqlBuilder":
            self.builder = MysqlBuilder(cfgfilepath, sesc, keytransformdicts)
        elif builder == "MysqlBuilderForToml":
            self.builder = MysqlBuilderForToml(cfgfilepath, sesc, keytransformdicts)
        elif builder == "MysqlBuilderForDicts":
            self.builder = MysqlBuilderForDicts(dicts, keytransformdicts)
        else:
            raise Exception("传入builder 参数有误")
        self.builder = self.builder.build_all().build_cursorclass(cursorsnum)
        self.moudle = self.builder.get_moudle()
        # 连接mysql通过唯一适配moudle
        # print(self.moudle.to_dict())
        self.mysql.link(self.moudle)

    @property
    def logger(self):
        if self._logger is None:
            self._logger = MLogger().streamlogger
        return self._logger

    @logger.setter
    def logger(self, value):
        assert isinstance(value, MLogger)
        self._logger = value

    # 更新数据库状态
    def ExeSqlListToDB(self, sSqlList, errExit=True, args=None):
        """
        该函数处理一个sql列表 没有返回并将列表置空
        :param sSqlList:
        :return:
        """
        dbMsg = None
        if not self.mysql:
            raise ValueError("database conn or database config must have to have one")
        if not self.mysql.is_ping():
            self.mysql.reConnect()
        cursor = self.mysql.get_new_cursor()
        errcount = 0
        successcount = 0
        for sql in sSqlList:
            try:
                self.logger.info(sql + ";" + str(args))
                cursor.execute(sql, args=args)
                successcount += 1
            except:
                errcount += 1
                self.mysql.commit()
                self.logger.error('*errSql:' + sql)
                dbMsg = '*updateError:' + traceback.format_exc()
                if errExit:
                    self.logger.error(dbMsg)
                    sys.exit(-1)
            if dbMsg:
                self.logger.error(dbMsg)
        self.mysql.commit()
        cursor.close()
        if errcount:
            return False, successcount, errcount
        else:
            return True, successcount, errcount

    def SelectFromDB(self, sSql, errExit=True, args=None):
        """
        使用sql语句查询并返回结果列表
        :param sql:
        :return:
        """
        if not self.mysql:
            raise ValueError("database conn or database config must have to have one")
        if self.mysql.is_ping():
            self.mysql.reConnect()
        cursor = self.mysql.get_new_cursor()
        try:
            self.logger.info(sSql + ";" + str(args))
            result = cursor.execute(sSql, args=args)
            rows = cursor.fetchall()
            return True, rows
        except:
            self.logger.error('* errSql:' + sSql)
            sMsg = '* errSynax:' + traceback.format_exc()
            if errExit:
                self.logger.error(sMsg)
                sys.exit(-1)
        finally:
            self.mysql.commit()
            cursor.close()
        if sMsg:
            self.logger.error(sMsg)
            return False, sMsg

    def SelectFromDBFetchOne_noyield(self, sSql, errExit=True, args=None):
        """
        使用sql语句查询并返回结果列表
        :param sql:
        :return:
        """
        if not self.mysql:
            raise ValueError("database conn or database config must have to have one")
        if self.mysql.is_ping():
            self.mysql.reConnect()
        cur = self.mysql.get_new_cursor()
        try:
            self.logger.info(sSql + ";" + str(args))
            result = cur.execute(sSql, args=args)
            return True, cur.fetchone()
        except:
            self.logger.error('* errSql:' + sSql)
            sMsg = '* errSynax:' + traceback.format_exc()
            if errExit:
                self.logger.error(sMsg)
                sys.exit(-1)
        finally:
            self.mysql.commit()
            cur.close()
        if sMsg:
            self.logger.error(sMsg)
            return False, sMsg

    def SelectFromDBFetchOne(self, sSql, errExit=True, args=None):
        """
        使用sql语句查询并返回结果列表
        :param sql:
        :return:
        """
        if not self.mysql:
            raise ValueError("database conn or database config must have to have one")
        if self.mysql.is_ping():
            self.mysql.reConnect()
        cur = self.mysql.get_new_cursor()
        try:
            self.logger.info(sSql + ";" + str(args))
            result = cur.execute(sSql, args=args)
            while True:
                row = cur.fetchone()
                if row is None:
                    return None
                else:
                    yield row
        except:
            self.logger.error('* errSql:' + sSql)
            sMsg = '* errSynax:' + traceback.format_exc()
            if errExit:
                self.logger.error(sMsg)
                sys.exit(-1)
        finally:
            self.mysql.commit()
            cur.close()
        if sMsg:
            self.logger.error(sMsg)
            return None

    def ExeSqlToDB(self, sSql, errExit=True, args=None):
        if not self.mysql:
            raise ValueError("database conn or database config must have to have one")
        if self.mysql.is_ping():
            self.mysql.reConnect()
        cur = self.mysql.get_new_cursor()
        try:
            self.logger.info(sSql + ";" + str(args))
            rows = cur.execute(sSql, args=args)
            self.mysql.commit()
            return True, rows
        except:
            self.mysql.rollback()
            sMsg = '* errSynax:' + traceback.format_exc()
            if errExit:
                self.logger.error(sMsg)
                sys.exit(-1)
        finally:
            self.mysql.commit()
            cur.close()
        if sMsg:
            self.logger.error(sMsg)
            return False, sMsg

    # 更新数据库状态
    def ExeSqlMany(self, sSql, args, errExit=True):
        """
        该函数处理一个sql列表 没有返回并将列表置空
        :param conn:
        :param sSqlList:
        :return:
        """

        if not self.mysql:
            raise ValueError("database conn or database config must have to have one")
        if self.mysql.is_ping():
            self.mysql.reConnect()
        cur = self.mysql.get_new_cursor()
        try:
            self.logger.info(sSql + ";" + str(args))
            rows = cur.executemany(sSql, args=args)
            self.mysql.commit()
            return True, rows
        except:
            self.mysql.rollback()
            self.logger.error('*errSql:' + sSql)
            dbMsg = '*updateError:' + traceback.format_exc()
            if errExit:
                self.logger.error(dbMsg)
                sys.exit(-1)
        finally:
            self.mysql.commit()
            cur.close()
        if dbMsg:
            self.logger.error(dbMsg)
            return False, dbMsg

    def escape_string(self, string):
        return self.mysql.escape(string)

    def pymysql_escape_string(self, strings):
        return self.mysql.escape(strings)

    def close(self):
        self.mysql.db.close()



import pydash

users = [
    {'name': 'Michelangelo', 'active': False},
    {'name': 'Donatello', 'active': False},
    {'name': 'Leonardo', 'active': True}
]
callback = lambda item: item["name"] == "Donatello"
pydash.find_index(users, callback)



"""
1.基本的读取配置文件

-read(filename) 直接读取ini文件内容

-sections() 得到所有的section，并以列表的形式返回

-options(section) 得到该section的所有option

-items(section) 得到该section的所有键值对

-get(section,option) 得到section中option的值，返回为string类型

-getint(section,option) 得到section中option的值，返回为int类型，还有相应的getboolean()和getfloat() 函数。

2.基本的写入配置文件

-add_section(section) 添加一个新的section

-set( section, option, value) 对section中的option进行设置，需要调用write将内容写入配置文件。
如果配置文件如下
[mysqllocalhost]
host = 127.0.0.1
port = 3306
user = root
passwd = xujiang1994323
db = test.txt
chartset = utf8mb4


[mysqlfdfs]
host = xujiang5.xicp.io
port = 3307
user = root
passwd = xujiang1994323
db = fdfs
chartset = utf8

"""
import configparser
# 读取配置文件
import sys
import traceback
import warnings
from abc import ABCMeta, abstractmethod
from collections import OrderedDict
from configparser import DEFAULTSECT


class ConfigParserMoudle():
    """
    config 解析的默认参数 防止以后需要使用 现在全部使用默认
    """

    def __init__(self):
        self.defaults = None
        self.allow_no_value = False
        self.delimiters = ('=', ':')
        self.comment_prefixes = ('#', ';')
        self.inline_comment_prefixes = None
        self.strict = True
        self.empty_lines_in_values = True
        self.default_section = DEFAULTSECT

    def to_dict(self):
        return self.__dict__

    def __str__(self):
        return str(self.__dict__)


class ConfigParserBuilderAbstract:
    __metaclass__ = ABCMeta

    @abstractmethod
    def build_defaults(self):
        pass

    @abstractmethod
    def build_allow_no_value(self):
        pass

    @abstractmethod
    def build_delimiters(self):
        pass

    @abstractmethod
    def build_comment_prefixes(self):
        pass

    @abstractmethod
    def use_inline_comment_prefixes(self):
        pass

    @abstractmethod
    def build_strict(self):
        pass

    @abstractmethod
    def use_empty_lines_in_values(self):
        pass

    @abstractmethod
    def build_default_section(self):
        pass

    @abstractmethod
    def get_moudle(self):
        pass


default_config_parser_moudle = ConfigParserMoudle()


class MyParser(configparser.ConfigParser):
    """
    继承了ConfigParser 方便以后重写
    """

    def __init__(self, configmoudle: ConfigParserMoudle):
        super(MyParser, self).__init__(**configmoudle.to_dict())


class IniConfig(object):

    def __init__(self, configpath=''):
        # 配置文件路径
        self.configpath = configpath
        # 指定的moudle格式 无特殊需求不用传入
        self.moudle = default_config_parser_moudle
        self.keep_keys_case = True
        self.encoding = "utf-8"

    def set_encoding(self, encoding="utf-8"):
        """
        设置读取文件编码
        :param encoding:
        :return:
        """
        self.encoding = encoding
        return self

    def set_keep_keys_case(self, keep_keys_case=True):
        """
        保持option的大小写
        :return:
        """
        self.keep_keys_case = keep_keys_case
        return self

    def set_config_path(self, configPath):
        """
        设置config文件路径
        :param configPath:
        :return:
        """
        self.configpath = configPath
        return self

    def set_configmoudle(self, configmoudle: ConfigParserMoudle):
        """
        设置 modudle,一般情况下不需要传入设置 使用默认设置即可
        :param configmoudle:
        :return:
        """
        self.moudle = configmoudle
        return self

    def read_config(self):
        """
        读取配置文件 这样cf本身就是一个字典
        :param encoding:
        :param configPath:
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        try:
            self.cf.read(self.configpath, encoding=self.encoding)
        except:
            print('读取配置失败:' + traceback.format_exc())
            sys.exit(-1)
        return self

    def read_config_string(self):
        """
        读取配置文件 将文件读成一个string传入
        :param cfstr:
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        try:
            with open(self.configpath, mode='rb') as f:
                content = f.read()
                if content.startswith(b'\xef\xbb\xbf'):  # 去掉 utf8 bom 头
                    content = content[3:]
            self.cf.read_string(content.decode(self.encoding))
        except:
            print('读取配置失败:' + traceback.format_exc())
            sys.exit(-1)
        return self

    def builder(self, is_read_str=False):
        self.cf = MyParser(self.moudle)
        if self.keep_keys_case:
            assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
            self.cf.optionxform = str
        if is_read_str:
            self.read_config_string()
        else:
            self.read_config()
        return self

    def get_cf(self):
        """
        获取config的对象对其操作
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        return self.cf

    def get_all_sections(self) -> list:
        """
        传入参数为configparser.ConfigParser
        返回为 list
        该函数得到所有sections 即配置文件中  [mysqlfdfs] 这种
        是配置文件的一级参数
        :param cf:
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        secs = self.cf.sections()
        if not secs:
            warnings.warn("sections is null,maybe your are cionfig path is wrong")
        return secs

    def get_sesc_options(self, sesc_str: str) -> list:
        """
        返回为 list
        该函数得到指定sections的options
        像 host 这种
        是配置文件的一级参数
        :param sesc_str:
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        opts = self.cf.options(sesc_str)
        if not opts:
            warnings.warn("opts is null,maybe your are sesc_str is wrong")
        return opts

    def get_items(self, sesc: str):
        """
        :return: [('host', '127.0.0.1'), ('port', '3306'), ('user', 'root'), ('passwd', 'xujiang1994323'), ('db', 'test.txt'), ('chartset', 'utf8mb4')]
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        kvs = self.cf.items(sesc)
        return kvs

    def get_value(self, sec_str: str, opt_key: str) -> str:
        """
        获取值 str
        :param cf:
        :param sec_str:
        :param opt_key:
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        str_value = self.cf.get(sec_str, opt_key)
        return str_value

    def get_value_int(self, sec_str: str, opt_key: str) -> int:
        """
        获取配置文件的值按Int形式
        :param cf:
        :param sec_str:
        :param opt_key:
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        int_value = self.cf.getint(sec_str, opt_key)
        return int_value

    def set_value(self, sec_str: str, opt_key: str, opt_value: str, filePath: str):
        """
        设置或者更新值
        :param cf:
        :param sec_str:
        :param opt_key:
        :param opt_value:
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        self.cf.set(sec_str, opt_key, opt_value)
        self.cf.write(open(filePath, "w"))

    def add_new_section(self, secs: str, filePath: str):
        """
        设置section
        :param secs:
        :param filePath:
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        self.cf.add_section(secs)
        self.cf.write(open(filePath, "w"))

    def get_config_dict(self) -> OrderedDict:
        """
        读取文件转dict
        :param cf:
        :return:
        """
        assert self.cf, AttributeError("还没有调用get_configparser为其赋值 请先调用该函数")
        d = OrderedDict(self.cf._sections)
        for k in d:
            d[k] = OrderedDict(d[k])
        return d



import logging
import traceback

from re_common.v2.baselibrary.utils.basepika import BasePika
from retry import retry

logging_logger = logging.getLogger(__name__)


class UseMq(object):

    def __init__(self, queue, qos=1):
        self.queue = queue
        self.qos = qos
        self.basepika = BasePika()
        self.basepika.set_default()
        self.basepika.connect()
        self.basepika.create_channel()
        self.basepika.queue_declare(queue=queue, durable=True)
        self.basepika.basic_qos(qos)
        self.properties = self.basepika.get_properties()

    def re_conn(self):
        """
        重新连接
        :return:
        """
        self.basepika.connect()
        self.basepika.create_channel()
        self.basepika.queue_declare(queue=self.queue, durable=True)
        self.basepika.basic_qos(self.qos)

    @retry(delay=5, backoff=2, max_delay=60 * 3, logger=logging_logger)
    def get_mq(self):
        try:
            if self.basepika.channel.is_closed:
                logging_logger.info("重连中......")
                self.re_conn()
                logging_logger.info("重连完成......")
            self.basepika.set_get_msg_callback(routing_key=self.queue, callback=self.callback, auto_ack=False)
            self.basepika.start_get_msg()
        except:
            traceback.print_exc()
            logging_logger.info("重连中......")
            self.re_conn()

    def callback(self, ch, method, properties, body):
        # print(type(body))
        # print(" [x] Received %r" % body)
        # body = body.decode()
        self.callback2(ch, method, properties, body)
        if self.basepika.auto_ack is False:
            self.basepika.basic_ack(ch, method)

    def callback2(self, ch, method, properties, body):
        pass

    @retry(delay=5, backoff=2, max_delay=60 * 3, logger=logging_logger)
    def send_mq(self, body, num=100):
        try:
            if self.basepika.get_queue_size(self.queue) < num:
                self.basepika.easy_send_msg(routing_key=self.queue,
                                            body=body,
                                            properties=self.properties)
                return True
            else:
                return False
        except:
            traceback.print_exc()
            logging_logger.info("重连中......")
            self.re_conn()
            return False

    def get_server_mq_num(self, num=100):
        if self.basepika.get_queue_size(self.queue) < num:
            return True
        else:
            return False

    def easy_send_mq(self, body):
        self.basepika.easy_send_msg(routing_key=self.queue,
                                    body=body,
                                    properties=self.properties)



from re_common.baselibrary import BaseDicts
from re_common.baselibrary.utils.mfaker import MFaker


def test_basedict_sortkeys():
    fake = MFaker()
    dicts = fake.create_data(MFaker.m_pydict, **fake.py_para())
    print(dicts)
    dicts2 = BaseDicts.sortkeys(dicts)
    print(dicts2)
    print(dicts)


def test_basedict_sortvalues():
    dicts = {'data': 'FDZqqOGNMyGJlNRoCsJd', 'participant': 'petersonadrienne@bennett.com',
             'often': 'KnJSSDeSTPboiwjSdGwR', 'friend': '1639', 'above': '8144', 'in': '1614'}
    dicts2 = BaseDicts.sortvalues(dicts)
    print(dicts2)




def compare(a: str, b: str):
    '''
    比较两个版本的大小，需要按.分割后比较各个部分的大小
    :return 返回较大的版本号
    '''
    lena = len(a.split('.'))  # 获取版本字符串的组成部分
    lenb = len(b.split('.'))
    a2 = a + '.0' * (lenb-lena)  # b比a长的时候补全a
    b2 = b + '.0' * (lena-lenb)
    for i in range(max(lena, lenb)):  # 对每个部分进行比较，需要转化为整数进行比较
        if int(a2.split('.')[i]) > int(b2.split('.')[i]):
            return a
        elif int(a2.split('.')[i]) < int(b2.split('.')[i]):
            return b
        else:						# 比较到最后都相等，则返回第一个版本
            if i == max(lena, lenb)-1:
                return a


def compare_bool(a: str, b: str):
    '''
    比较两个版本的大小，需要按.分割后比较各个部分的大小
    :return 如果 a >= b 返回true
    '''
    lena = len(a.split('.'))  # 获取版本字符串的组成部分
    lenb = len(b.split('.'))
    a2 = a + '.0' * (lenb-lena)  # b比a长的时候补全a
    b2 = b + '.0' * (lena-lenb)
    for i in range(max(lena, lenb)):  # 对每个部分进行比较，需要转化为整数进行比较
        if int(a2.split('.')[i]) > int(b2.split('.')[i]):
            return True
        elif int(a2.split('.')[i]) < int(b2.split('.')[i]):
            return False
        else:						# 比较到最后都相等，则返回第一个版本
            if i == max(lena, lenb)-1:
                return True



"""
来自于 PEP572

在 python 3.8 中获取正式支持

https://docs.python.org/3/whatsnew/3.8.html

:= (由于它与海象的眼睛和象牙很像，因此被亲切地称为“海象操作员” )

"""

# 在此示例中，赋值表达式有助于避免调用 len()两次：
import re
import string

a = "adahfhoifoigoiwfhoiwg"
if (n := len(a)) > 10:
    print(f"List is too long ({n} elements, expected <= 10)")

# 在正则表达式匹配期间会产生类似的好处，其中需要两次匹配对象，一次是测试是否发生匹配，另一次是提取子组：
discount = 0.0
advertisement = "a 50% discount"
if mo := re.search(r'(\d+)% discount', advertisement):
    discount = float(mo.group(1)) / 100.0
    print(discount)


# 该运算符还对while循环有用，该循环计算一个值来测试循环终止，然后在循环主体中再次需要相同的值：
# Loop over fixed length blocks
f = open("./__init__.py", 'rb')
while len(block := f.read(256)) != 0:
    print(len(block))

# 另一个具有启发性的用例出现在列表理解中，其中表达式主体中还需要在过滤条件下计算出的值：
allowed_names = list(string.ascii_letters)
names = list(string.ascii_letters)
print([clean_name for name in names if (clean_name := name) in allowed_names])